
#ifndef _sram_h_
	#define _sram_h_
	
	#include <system.h>
	#include <string.h>
	#include <stdio.h>
	#include <stdlib.h>
	#include "config.h"
	#include "spi.h"
	
	#define SRAM_MAX_SIZE 0x7FFF
	/* in config.h you must add:
		#include "spi.h" //sram uses SPI, include your SPI module
		#define RAM_cs   portx.x    //chip select active low
		#define RAM_hold portx.x    //hold active low
		
		//the tristate registers to select cjip
		#define RAM_tris_cs   trisx.x
		#define RAM_tris_hold trisx.x
		
		//SRAM uses SPI bus to read/write data, define how to read /write to the bus:
		//you can use #define like this:
		#define sram_spi_transmit(a) spi_transmit(a)
		#define sram_spi_read() spi_read()
		#define sram_spi_transmit_array(a,b) spi_transmit_array(a,b)
		#define sram_spi_read_array(a,b) spi_read_array(a,b) //notice: spi_read_array() should return the number of bytes read from the bus
		
		
		
	//or use inline functions:
	inline void sram_spi_transmit(unsigned char data){
		spi_transmit(data);
	}
	inline unsigned char sram_spi_read(){
		return spi_read();
	}
	inline void sram_spi_transmit_array(unsigned char * data, unsigned int len){
		spi_transmit_array(data, len);
	}
	inline unsigned int sram_spi_read_array(unsigned char * data, unsigned int len){
		spi_read_array(data,len);
		return len;
	}
	*/

	void sram_open(unsigned int address, bool read);//enables the chip and let's it point to address
	void sram_write_byte(unsigned char byte); //can be called after sram_write(address,false). It writes a byte to address, calling again will write to address+1,+2,...
	unsigned char sram_read_byte(); //can be called after sram_open(address,true). It reads byte at address, calling again will read at address+1, address+2,...
	void sram_close(); //to be called after each sram_open() it disables the chip (cs=1)
	void sram_init(); //initialize (call when pic starts)
	unsigned char sram_readstatus(); //reads the status word
	void sram_write(unsigned int address,unsigned char * data,unsigned char len);//writes len bytes to address
	void sram_write(unsigned int address,unsigned char * data, unsigned int len);
	void sram_write_string(unsigned int address, unsigned char * data);//writes a string terminated with null character to address
	void sram_read(unsigned int address, unsigned char * data, unsigned char len); //reads len bytes starting at address into the data buffer (buffer must be long enough)
	void sram_read(unsigned int address, unsigned char * data, unsigned int len);
	void sram_read_string(unsigned int address, const unsigned char * data);//reads a null-terminated string from address (data buffer must be long enough)
	unsigned int sram_read_int(unsigned int start); //reads an integer at address start, stops reading when it encounters a non numerical character
	int sram_find_array(unsigned int Start, unsigned int Stop, unsigned char * Sequence, size_type SequenceCnt, bool CaseSensetive);//searches for a string (find) in the memory starting at start and looking until stop, sequencelen is the lenght of the string to find.
																													//Returns the address where the sequence starts, -1 if it was not found
	int  sram_find_string(unsigned int start, unsigned int stop, unsigned char * find, bool CaseSensetive);//same as sram_find_array but requires find to be a null terminated string 
	void sram_copy(unsigned int src, unsigned int dst, unsigned int len); //copy data from src address (in serial ram) to destination (in serial ram)
	void sram_fill_val(unsigned int start, unsigned int len, unsigned char val); //fills len bytes with val starting at address start
	bool sram_push(unsigned char * buffer, size_type len); //writes len bytes from buffer to the sram stack, returns true if success buffer can be NULL
	void sram_pop(unsigned char * buffer, size_type len);//removes len bytes from the sram stack, saving them in the buffer buffer can be NULL
	unsigned int sram_get_stack_ptr();//returns a ptr to the next free stack byte
	void sram_set_stack_ptr(unsigned int ptr); //sets the stack pointer
	void sram_set_stack_start(unsigned int start);//sets the start of the stack and initializes the stack pointer to this location
#endif

