
#include "http.h"

const rom char * httpproto = "http://";

typedef enum {
  HTTP_IDLE,
  HTTP_CONNECTED,
  HTTP_END,
  HTTP_DISCONNECT
} HTTPState;

typedef struct {
	TCP_SOCKET Socket;
	HTTPState State;
} HTTPServerConnection;

void HTTPProcess (HTTPServerConnection * h);
void http_handle_connection (HTTPServerConnection * h);

static HTTPServerConnection HTTPConnections[HTTP_MAX_REQUEST];

//parses a string until it finds no white space anymore (' ')
unsigned char * ParseSpace(unsigned char *str){
	while (*str==' ' && *str!=0)str++;
	return str;
}



//Starts the httpserver
void HTTPInitServer(unsigned int port){
	if (port==0) port=80;
	for (unsigned char i=0;i<HTTP_MAX_REQUEST;i++){
		HTTPConnections[i].Socket = TCPOpen(NULL,NULL, TCP_OPEN_SERVER, port);
		HTTPConnections[i].State=HTTP_IDLE;
	}
}

//does a http task
void HTTPTask(){
	for (unsigned char i=0;i<HTTP_MAX_REQUEST;i++){
		http_handle_connection (&HTTPConnections[i]);
	}
}

void http_handle_connection (HTTPServerConnection * h){
	if (h->Socket!=INVALID_SOCKET){
		if (!TCPIsConnected(h->Socket)){
			h->State=HTTP_IDLE;
			return;
		}
		
		switch(h->State){
		case HTTP_IDLE:
			if (TCPIsConnected(h->Socket)) h->State=HTTP_CONNECTED;
			break;
		case HTTP_CONNECTED:
			int pos=TCPFindString(h->Socket, "\r\n\r\n",false); //see if we received the header request
			if (pos > 0){
				HTTPProcess(h);
				TCPFlush(h->Socket);
				h->State=HTTP_END; //request was processed
			}
			break;
		case HTTP_END:
			TCPDiscard(h->Socket);
			h->State=HTTP_DISCONNECT;
		case HTTP_DISCONNECT:
			TCPDisconnect(h->Socket);
			h->State=HTTP_IDLE;
			break;
		}
	}
}

//process the headers and send the respons
void HTTPProcess (HTTPServerConnection * h){
	unsigned char headerline[HTTP_MAX_HEADER_SIZE];
	unsigned char method[HTTP_METHOD_SIZE];
	unsigned char url[HTTP_URL_SIZE];
	HTTPParameter pars[HTTP_MAX_PAR];

	int pos = TCPFindString(h->Socket, "\r\n",false);
	if (pos-1>0){
		pos = min(HTTP_MAX_HEADER_SIZE-1,pos-1); //holds the end of the header size
		TCPGetArray(h->Socket,headerline,pos);
		//unsigned char * endofline = strrchr(headerline,' ');
		//if (endofline!=NULL)*endofline='\0'; //end string with 0 before "HTTP/1.x\r\n"
		unsigned char parcount = HTTPParseRequest(headerline,method,url,pars,HTTP_MAX_PAR);
		if (strlen(url)!=0){
				TCPPutString(h->Socket,"HTTP/1.1 200 OK\r\n\r\n<html><head><title>this is a test</title></head>");
				TCPPutString(h->Socket,"<body>");
				TCPPutString(h->Socket,"Request=");
				TCPPutString(h->Socket,url);
				TCPPutString(h->Socket,"Method=");
				TCPPutString(h->Socket,method);
				TCPPutString(h->Socket,"parcount=");
				TCPPut(h->Socket, '0'+parcount);
				TCPPutString(h->Socket,"<br>\r\n");
				for (unsigned char i=0;i<parcount;i++){
					TCPPutString(h->Socket,"Parameter: ");
					TCPPutString(h->Socket,pars[i].name);
					TCPPutString(h->Socket,"=");
					TCPPutString(h->Socket, pars[i].val);
					TCPPutString(h->Socket, "<br>\r\n");
				}
				TCPPutString(h->Socket,"</body>");
		}else{
			TCPPutString(h->Socket,"url=0");
		}
	}else{
		TCPPutString(h->Socket,"pos=0");
	}

}


//parses an http request
//returns number of parameters found, method, filename and all parameters and their values
unsigned char HTTPParseRequest(unsigned char *str,unsigned char * method, unsigned char * file,HTTPParameter * pars, unsigned char argc){
	unsigned int argindex = 0,i=0;
	if (*str!=0) str = ParseString(str, method," ",HTTP_METHOD_SIZE); //GET or POST
	if (*str!=0) str++; //skip the ' '
	if (*str!=0) str = ParseString(str, file, "? ",HTTP_URL_SIZE); //get the file name this ends with a ? or a space
	if (*str=='?'){ //yes there are parameters
		if (*str!=0) str++; //skip the '?'
		while (str!=NULL && *str!=0 && i<argc){
			str= ParseHTTPParameter(str, pars);
			i++;
			pars++;
		}
	}
	return i;
}

//cmdsize determines the max size of a parameter and value (=buffer size)
unsigned char * ParseHTTPParameter(unsigned char * str, HTTPParameter * par){
	//unsigned char * amp, * equal;
	//while ((*str=='?' || *str=='&') && *str!=0) str++; //skip all "?" and &
	str = ParseString(str,par->name, "&= ",HTTP_PAR_SIZE); //read in the parameter name, stop parsing as soon as we find a " ", & or =
	if (*str!=0 && *str=='=' && str[1]!='&') str = ParseString(str,par->val,"&= ",HTTP_VAL_SIZE); //read in the parameter value, stop parsing as soon as we find a " ", & or =
	else par->val[0]=0;
	if (*str==' ') return NULL; //end of url
	return str;
}


//parses a string from a large string determined by any character in delims, the string found is set in dst
//returns pointer where the string ends
// example:
// "   this is a test" and delim= " " 
// dst= "this\0" and a pointer to the ' ' character after "this"
unsigned char * ParseString(unsigned char * str, unsigned char * dst, unsigned char * delims, size_type dstlen){
	size_type size=1;//start at one cuz we need space for at least one "\0" character
	while (*str!=0 && strchr(delims,*str)!=NULL) str++;
	while (*str!=0 && strchr(delims,*str)==NULL){
		if (dst!=NULL && size<dstlen){
			*dst = *str;
			dst++;
			size++;
		}
		str++;
	}
	if (dst!=NULL)*dst=0;
	return str;
}

bool HTTPDownloadFile(unsigned char * url, unsigned char * method, HTTPRespons * result){
	if (strstr(url,httpproto)!=0) url+=strlen(httpproto);
	unsigned char * file = strchr(url,'/');
	unsigned char * strport = strchr(url,':');
	unsigned char serverlen;
	unsigned char buffer[HTTP_MAX_SERVER_SIZE];
	unsigned int port;
	unsigned long maxtime = rtc_get_time() + HTTP_DOWNLOAD_TIMEOUT;
	
	result->headerstart=sram_get_stack_ptr(); //tell where we start in the sram
	
	if (strport!=NULL){
		port = strtoi(strport+1,NULL,10);
		serverlen = strport - url;
	}else{
		if (file!=NULL){
			serverlen = file - url;
		}else{
			serverlen = strlen(url);
		}
		port=80;
	}
	
	if (serverlen > HTTP_MAX_SERVER_SIZE) return false;
	strncpy(buffer,url,serverlen); //copy the DNS server name to the temporary buffer string
	
	IP_ADDR ServerIP;
	
	if (DNSResolveBlock(buffer, DNS_TYPE_A,&ServerIP , &maxtime)){
		TCP_SOCKET client = TCPOpen(&ServerIP,NULL,TCP_OPEN_CLIENT_IP,port);
		if (client!=INVALID_SOCKET){
			while (!TCPIsConnected(client) && rtc_get_time() < maxtime) network_handle_tasks(); //wait for connection
			if (TCPIsConnected(client)){
				if (method!=NULL){
					if (!TCPPutStringBlock(client, method,&maxtime)) return false;
				}else{
					if (!TCPPutStringBlock(client, "GET",&maxtime)) return false;
				}
				if (!TCPPut(client, ' ')) return false;
				if (file!=NULL){
					if (!TCPPutStringBlock(client, file, &maxtime)) return false;
				}else{
					TCPPut(client, ' ');
				}
				if (!TCPPutStringBlock(client, " HTTP/1.0\r\nConnection: Close\r\n", &maxtime)) return false;
				if (!TCPPutStringBlock(client, "Host: ", &maxtime)) return false;
				if (!TCPPutStringBlock(client, buffer,&maxtime)) return false;
				if (!TCPPutArrayBlock(client, "\r\n\r\n",4,&maxtime)) return false;
				TCPFlush(client);
				
				
				//now we turn the communication, we wait for the host to finish sending the headers
				unsigned char read;
				
				while (TCPIsConnected(client) && rtc_get_time < maxtime){
					network_handle_tasks();//handle network tasks important! must be placed here else we might lose a few bytes transmitted just before socket closed
					read = TCPGetArray(client, buffer ,HTTP_MAX_SERVER_SIZE); //read in data (max 32 bytes) until tcp fifo is empty
					while (read!=0){
						if (!sram_push(buffer, read))goto failed;  //now push the data into the sram module if this fails then we are out of ram
						read = TCPGetArray(client, buffer ,HTTP_MAX_SERVER_SIZE); //read in data (max 32 bytes)
					}		
				}
				//check if time out, process the content
				if (rtc_get_time() < maxtime){//the file was downloaded within the timeout
					//search for header end
					result->respons = sram_read_int(result->headerstart+10); //read in the 200 OK or other error
					int pos =sram_find_string(result->headerstart, sram_get_stack_ptr(), "\r\n\r\n",false);
					if (pos!=-1){
						result->headerlen = pos - result->headerstart+4;
						result->filesize = sram_get_stack_ptr() - pos -4;
						return true;
					}
				}
			}
		}
	}
	
failed:
	sram_set_stack_ptr(result->headerstart); //reset the stack pointer
	return false;
}