#include <system.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
/*
		PIC18F27J53 & Sourceboost Ethernet example using ENC28J60
		(c) Tom 5-6-2011 
		(current website: www.dfzone.be)
		do whatever you like with this sourcecode 
		
		pins used:
		RB7 SCK (SPI bus)
		RB6 SDO (SPI bus)
		RB1 SDI (SPI bus)
		RB0 Output for LED (blinks once a second for indicating controller is running)
		RA6 & RA7 12MHz crystal (internal clock is 48MHz)
		RA0 Chip select for serial RAM (23K256)
		RA1 Chip select for ENC28J60
		Also see the schematic PDF file
		RC6  RX1 pin
		RC7  TX1 pin (here the controller will send debug information at 9600baud 8 databit 1 stop bit no parity)
		
		To add the code to another project you should:
		add:
		 -stackconf.h to the root of your project folder, edit it to fit your requirements (adjust the spi, sram & rtc routines in it)
		 -include stackmain.h located in the stack folder
		 -other stack files might be added to the project when needed
		
*/

/*
IMPORTANT:
the debug configuration bit was accidently removed
in case of errors you need to add this code to PIC18F27J53.tdf file located in <sourceboost installation>\config:
TargetConfig DEBUG
{
    // Background Debugger Enable:
    ConfigRegAddr = 0x0001FFF8;
    ConfigBitMask = 0b10000000;
    ConfigSettings =
        0b00000000, "ON","Enabled",
        0b10000000, "OFF","Disabled";
}
//also add:
TargetConfig LS48MHZ
{
    // 24/48MHz USB clock select:
    ConfigRegAddr = 0x0001FFFF;
    ConfigBitMask = 0b00001000;
    ConfigSettings =
        0b00001000, "ON","48-MHz system clock is expected divide-by-8 generates low-speed USB clock",
        0b00000000, "OFF","24-MHz system clock is expected divide-by-4 generates low-speed USB clock";
}
*/


// Config for 12Mhz crystal CPU clock at 48Mhz (96Mhz/2) PLL
#pragma CLOCK_FREQ 48000000
#pragma config DEBUG = OFF, XINST = OFF, STVREN = ON, CFGPLLEN = ON, PLLDIV = 3, WDTEN = OFF //  CONFIG1L no debugger, extended instr. off, stack overflow/underflow off,  PLL on, Divide by 3 (12 MHz osc), WAD OFF
#pragma config CPUDIV = OSC1,CP0 = OFF //cpu div = 1, code protect off
#pragma config IESO = OFF, FCMEN = OFF, CLKOEC = OFF, SOSCSEL = RESERVED,  OSC = HSPLL // HS+PLL, USB-HS+PLL
#pragma config WDTPS = 16384
#pragma config DSWDTPS=512,DSWDTEN = OFF, DSBOREN = OFF, RTCOSC = INTOSCREF, DSWDTOSC = INTOSCREF // 1:32768 RTCC uses INTRC DSWDT uses INTRC
#pragma config MSSP7B_EN = MSK7, ADCSEL = BIT12, IOL1WAY = OFF
#pragma config WPCFG = OFF  // The IOLOCK bit (PPSCON<0>) can be set and cleared as needed
#pragma config LS48MHZ = SYS48X8, WPEND = PAGE_WPFP, WPDIS = OFF

#include "config.h"
#include "sp.h"
#include "spi.h"
#include "SRAM.h"
#include "rtc.h"
#include "stack\stackmain.h"
#include "http.h"

void startUDPtest();
void custom_network_task();
void startTCPtest();
void initnetwork();
UDP_SOCKET UDPTest=INVALID_UDP_SOCKET;
TCP_SOCKET TCPTest=INVALID_SOCKET;

/*
This is some stuff i used to debug because i had problems of the code getting stuck
before every while/do i placed pushcs(#) and after the loop popcs() 
when the loop get's stuck forever a watchdog will occur and we can print the 'callstack' to see
where we got stuck
callstack defs:
0x01 network_handle_tasks();
0x02 udpputarray
0x03 UDPFlush
0x04 TCPTick
0x05 UDPTask
0x06 MACGetFreeRxSize
0x07 MACFlush
0x08 while(!(ReadETHReg(ENC_EIR) & (1<<ENC_TXERIF | 1<<ENC_TXIF)));
0x09 while(len){
0x0A while(ChunkLen){
0x0B ReadPHYReg
0x0C WritePHYReg
0x0D while(!IPIsTxReady()); in TCP.c
0x0E CalcIPBufferChecksum(WORD len) in IP.c
0x0F while(!MACIsTxReady()); in ARPPut();
0x10 UDPIsPutReady in nbns.c
0x11 MACIsLinked
*/
/*unsigned char callstack[200];
unsigned char callstackcount;

void pushcs(unsigned char sub){
	callstack[callstackcount]=sub;
	if (callstackcount!=199)callstackcount++;
	else callstack[199]=0xFF;//overflow
}

void popcs(){
	if(callstackcount!=0)callstackcount--;
}*/

void interrupt(){
	if (pir2.TMR3IF){
		rtc_handle_timer();
		pir2.TMR3IF=0;
	}
}

void interrupt_low(){
	nop();
}

//this function initializes the stack
void initnetwork(){
	MAC_ADDR mac;
	IPV4_CONFIG networkconfig;
	mac.v[0]=2;
	mac.v[1]=3;
	mac.v[2]=4;
	mac.v[3]=5;
	mac.v[4]=6;
	mac.v[5]=2;
	networkconfig.IPV4Addr.v[0]=0;
	networkconfig.IPV4Addr.v[1]=0;
	networkconfig.IPV4Addr.v[2]=0;
	networkconfig.IPV4Addr.v[3]=0;
	networkconfig.Mask.v[0]=0;
	networkconfig.Mask.v[1]=0;
	networkconfig.Mask.v[2]=0;
	networkconfig.Mask.v[3]=0;
	networkconfig.Gateway.v[0]=0;
	networkconfig.Gateway.v[1]=0;
	networkconfig.Gateway.v[2]=0;
	networkconfig.Gateway.v[3]=0;
	networkconfig.Flags.bIsDHCPEnabled=true; //enable DHCP
	
	network_init(& mac,& networkconfig); //this inits all stack modules that are enabled in stackconf.h (ENC28J60, IP, ARP, TCP, UDP, DNS,...) also configures the mac & ip address
	network_set_netBIOSname("PIC18F27J53"); //set the netbiosname
	
	while (network_in_configmode()){ //wait until we have an IP from the DHCP server (you should add some code to skip this when it fails)
		clear_wdt();
		network_handle_tasks();
	}
	
	startUDPtest(); //start the UDP test
	startTCPtest(); //start a TCP server that will echo all data received at port 23
	HTTPInitServer(80); //start the http server
}

//this opens a socket at UDP port  5250 & 17478 and sends a UDP packet to 192.168.1.78:17478 every +/-10 seconds
void startUDPtest(){ 
	unsigned long maxtime  =rtc_get_time()+10; //max 10 seconds then we stop arp resolving
	NODE_INFO remotePC;
	remotePC.IPAddr.v[0]=192; //the remote PC enter IP here
	remotePC.IPAddr.v[1]=168;
	remotePC.IPAddr.v[2]=1;
	remotePC.IPAddr.v[3]=78;
	
	if (ARPResolveBlock(&remotePC.IPAddr, &remotePC.MACAddr,&maxtime)){ //before we can set up the UDP port we need to find the mac address
		UDPTest= UDPOpen (5250, &remotePC , 17478);
	}else{
		UDPTest = INVALID_UDP_SOCKET;
		serial_send_string("Unable to find MAC of ");
		serial_send_ip((unsigned char *)remotePC.IPAddr);
		serial_send_string("\r\n");
	}
}

//this opens a server at TCP port 23
void startTCPtest(){
	TCPTest = TCPOpen(NULL,NULL,TCP_OPEN_SERVER,23);
	if (TCPTest!=INVALID_SOCKET){
		serial_send_string("TCP server at port 23 started");
	}else{
		serial_send_string("failed to start tcp server at port 23");
	}
}

//checks if UDP data was received at port 5250 and every 10 seconds it will send a packet to 192.168.1.78:17478
void handleUDPtest(){
	static unsigned long ptime;
	if (UDPTest!=INVALID_UDP_SOCKET){
		if (UDPIsGetReady(UDPTest)!=0){
			clear_wdt();
			unsigned char buffer[50];
			unsigned char read;
			serial_send_string("Get UDP data\r\n");
			read = UDPGetArray(buffer,50);
			serial_send_string("UDPIn:");
			serial_send_data(buffer,read);
			serial_send_string("\r\n");
			UDPDiscard();
		}
		if (UDPIsPutReady(UDPTest) && rtc_get_time()-ptime>=10){
			unsigned char data[]="UDP test";
			UDPPutArray(data,strlen(data));
			UDPFlush();
			ptime=rtc_get_time();
		}
	}
}

void handleTCPtest(){
	if (TCPTest!=INVALID_SOCKET && TCPIsConnected(TCPTest)){ //test if there is connection
		unsigned char buffer[32]; 
		if (TCPIsGetReady(TCPTest)!=0){
			unsigned char read = TCPGetArray(TCPTest, buffer, 32); //read in the received data
			while (read!=0 && TCPIsPutReady(TCPTest)>=read){ //loop until all data is processed or the send buffer is full
				TCPPutArray(TCPTest, buffer, read);
				read = TCPGetArray(TCPTest, buffer, 32);
			}
			TCPFlush(TCPTest); //flush the send buffer
		}
	}
}

//after a call to network_handle_tasks(); the stack will call this function
//inside this function any server (TCP/UDP) should check if it received a packet!
void custom_network_task(){
	handleUDPtest();
	HTTPTask(); //handles the small http server
	handleTCPtest();
}


void main(){
	ancon0 = 0b11111111; //turn off all analog inputs
	ancon1 = 0b00011111; //turn off all analog inputs
	cm1con = 7; //turn off comparators
	cm2con = 7;
	cm3con = 7;
	trisb.0 = 0;
	portb.0 = 1;
	
	//turn on interrupts
	intcon.GIE=1;
	intcon.PEIE=1;
	
	rtc_init(); //start the "cpu clock" this code handles the timing of a 'real time' clock which can hold seconds since 1970 and also can be used for TCP/IP stack timings
	serial_init(); //init the serial port 1
	serial_send_string("INIT...\r\n"); //say that we are running
	
	wdtcon.SWDTEN=1; //enable the watchdog (which is configured to reset the chip after several seconds)
	
	if (rcon.TO==0) serial_send_string("WATCHDOG\r\n");
	/*if (rcon.TO==0){//watchdog was triggered write the "call stack" so we know where we got stuck (this was used for debugging only once, it showed the code got stuck reading PHY registers, problem is now fixed)
		clear_wdt();
		serial_send_string("WATCHDOG\r\n");
		for (unsigned char c=0;c<callstackcount;c++){
			serial_send_hex(callstack[c]);
			serial_send_string("\r\n");
			clear_wdt();
		}
		serial_send_string("END\r\n");
	}*/
	clear_wdt();

	spi_init(); //init the spi interface
	sram_init(); //init the sram
	
	//do the spi ram test, if SPI interface is not connected properly this code will fail and tell you
	serial_send_string("SPI & SRAM INIT OK, Performing memory test\r\n");
	
	unsigned char SomeData[250];
	for (unsigned char j=0;j<250;j++) SomeData[j]=j;
	sram_write(0,SomeData,250); //write data
	for (j=0;j<250;j++) SomeData[j]=0;
	sram_read(0,SomeData,250);
	for (j=0;j<250;j++){
		if (SomeData[j]!=j){
			serial_send_string("Error address: ");
			serial_send_hex(j);
			serial_send_string("\r\n");
		}
	}
	clear_wdt();
	serial_send_string("mem test done.\r\n");
	//end SPI ram test
	
	
	//start the network module and the stack
	initnetwork();
	clear_wdt();
	
	
	//example of how you could resolve a DNS name (google)
	unsigned long maxtime  = rtc_get_time() + 120; //we will spend a max of 120 seconds resolving the host name
	IP_ADDR googleip; //holds the IP from the domain name
	
	if (DNSResolveBlock("google.com", DNS_TYPE_A, &googleip,&maxtime)){
		serial_send_string("google.com has ip");
		serial_send_ip((unsigned char *)&googleip);
		serial_send_string("\r\n");
	}else{
		serial_send_string("unable to resolve DNS name");
	}
	//end example DNS resolve
	
	//example of downloading a file
	HTTPRespons result;
	
	if (HTTPDownloadFile("http://www.railtime.be/mobile/", "GET", &result)){ //downloads a small html file from the internet, returns true when finished successfully
		serial_send_string("Donwload OK:\r\n"); //tell the size,...
		serial_send_int(result.respons);
		serial_send_string("\r\n");
		serial_send_int(result.headerlen);
		serial_send_string("\r\n");
		serial_send_int(result.filesize);
		serial_send_string("\r\ncontent:\r\n");
		sram_open(result.headerstart+result.headerlen,true); //print the content of the file from external ram to the uart
		for(unsigned int l=0;l<result.filesize;l++){
			unsigned char b = sram_read_byte();
			serial_send_data(&b,1);
		}
		sram_close();
		sram_pop(NULL,result.headerlen+result.filesize);//remove the file from the external ram if not needed anymore
	}else{
		serial_send_string("Download failure\r\n");
	}
	//end download example 
	
	
	unsigned int i=0;
	bool reinitialize=false;
	unsigned long ptime=0;
	while(1){//endless loop
		if (ptime!=rtc_get_time()){ //portb.0 will toggle every second to show the cpu is still running
			portb.0 = ~ portb.0;
			ptime = rtc_get_time();
		}
		clear_wdt();
		if (MACIsLinked()){ //if the network link goes down we should restart the stack
			if (reinitialize){
				initnetwork();
				reinitialize=false;
			}
			clear_wdt();
			network_handle_tasks(); //do the network tasks
			clear_wdt();
		}else{
			serial_send_string("network link down!");
			reinitialize =true;
			clear_wdt();
		}
	}
	
}
