/*********************************************************************
 *
 *                  TCP Module Defs for Microchip TCP/IP Stack
 *
 *********************************************************************
 * FileName:        TCP.h
 * Dependencies:    StackTsk.h
 * Processor:       PIC18, PIC24F, PIC24H, dsPIC30F, dsPIC33F, PIC32
 * Compiler:        Microchip C32 v1.05 or higher
 *					Microchip C30 v3.12 or higher
 *					Microchip C18 v3.30 or higher
 *					HI-TECH PICC-18 PRO 9.63PL2 or higher
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * Copyright (C) 2002-2009 Microchip Technology Inc.  All rights
 * reserved.
 *
 * Microchip licenses to you the right to use, modify, copy, and
 * distribute:
 * (i)  the Software when embedded on a Microchip microcontroller or
 *      digital signal controller product ("Device") which is
 *      integrated into Licensee's product; or
 * (ii) ONLY the Software driver source files ENC28J60.c, ENC28J60.h,
 *		ENCX24J600.c and ENCX24J600.h ported to a non-Microchip device
 *		used in conjunction with a Microchip ethernet controller for
 *		the sole purpose of interfacing with the ethernet controller.
 *
 * You should refer to the license agreement accompanying this
 * Software for additional information regarding your rights and
 * obligations.
 *
 * THE SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT
 * WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT
 * LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * MICROCHIP BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF
 * PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
 * BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE
 * THEREOF), ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER
 * SIMILAR COSTS, WHETHER ASSERTED ON THE BASIS OF CONTRACT, TORT
 * (INCLUDING NEGLIGENCE), BREACH OF WARRANTY, OR OTHERWISE.
 *
 *
 * Author               Date    	Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Nilesh Rajbharti     5/8/01  	Original        (Rev 1.0)
 * Howard Schlunder		11/30/06	See "TCPIP Stack Version.txt" file
 ********************************************************************/
#ifndef __TCP_HITECH_WORKAROUND_H
	#define __TCP_HITECH_WORKAROUND_H
	
	#include "stackmain.h"
	/****************************************************************************
	  Section:
		Type Definitions
	  ***************************************************************************/
	
	// A TCP_SOCKET is stored as a single BYTE
	typedef BYTE TCP_SOCKET;
	
	#define INVALID_SOCKET      (0xFE)	// The socket is invalid or could not be opened
	#define UNKNOWN_SOCKET      (0xFF)	// The socket is not known
	#define TCP_ADJUST_GIVE_REST_TO_RX	0x01u	// Resize flag: extra bytes go to RX 
	#define TCP_ADJUST_GIVE_REST_TO_TX	0x02u	// Resize flag: extra bytes go to TX
	#define TCP_ADJUST_PRESERVE_RX		0x04u	// Resize flag: attempt to preserve RX buffer
	#define TCP_ADJUST_PRESERVE_TX		0x08u	// Resize flag: attempt to preserve TX buffer
	/*#define TCP_FROM_PIC_RAM 0x01u
	#define TCP_TO_PIC_RAM 0x02u
	#define TCP_TO_ETH_RAM 0x04u
	#define TCP_FROM_ETH_RAM 0x08u
	#define TCP_RAM_TO_RAM 0x16u*/
	#define TCP_OPEN_CLIENT_HOST 0x01
	#define TCP_OPEN_CLIENT_IP 0x02
	/****************************************************************************
	  Section:
		State Machine Variables
	  ***************************************************************************/
	
	// TCP States as defined by RFC 793
	typedef enum {
		TCP_GET_DNS_MODULE,		// Special state for TCP client mode sockets
		TCP_DNS_RESOLVE,		// Special state for TCP client mode sockets
		TCP_GATEWAY_SEND_ARP,	// Special state for TCP client mode sockets
		TCP_GATEWAY_GET_ARP,	// Special state for TCP client mode sockets
	
		TCP_LISTEN,				// Socket is listening for connections
		TCP_SYN_SENT,			// A SYN has been sent, awaiting an SYN+ACK
		TCP_SYN_RECEIVED,		// A SYN has been received, awaiting an ACK
		TCP_ESTABLISHED,		// Socket is connected and connection is established
		TCP_FIN_WAIT_1,			// FIN WAIT state 1
		TCP_FIN_WAIT_2,			// FIN WAIT state 2
		TCP_CLOSING,			// Socket is closing
	//	TCP_TIME_WAIT, state is not implemented
		TCP_CLOSE_WAIT,			// Waiting to close the socket
		TCP_LAST_ACK,			// The final ACK has been sent
		TCP_CLOSED,				// Socket is idle and unallocated
	
		TCP_CLOSED_BUT_RESERVED	// Special state for TCP client mode sockets.  Socket is idle, but still allocated pending application closure of the handle.
	} TCP_STATE;
	
	typedef enum {
		SSL_NONE = 0,			// No security is enabled
		SSL_HANDSHAKING,		// Handshake is progressing (no application data allowed)
		SSL_ESTABLISHED,		// Connection is established and secured
		SSL_CLOSED				// Connection has been closed (no applicaiton data is allowed)
	} SSL_STATE;
	
	/****************************************************************************
	  Section:
		TCB Definitions
	  ***************************************************************************/
	
	#define bServer 15
	#define bTimerEnabled 14
	#define bTimer2Enabled 13
	#define bDelayedACKTimerEnabled 12
	#define bOneSegmentReceived 11
	#define bHalfFullFlush 10
	#define bTXASAP 9
	#define bTXASAPWithoutTimerReset 8
	#define bTXFIN 7
	#define bSocketReset 6
	#define bSSLHandshaking 5
	
	
	// TCP Control Block (TCB) stub data storage.  Stubs are stored in local PIC RAM for speed.
	typedef struct {
		PTR_BASE bufferTxStart;		// First byte of TX buffer
		PTR_BASE bufferRxStart;		// First byte of RX buffer.  TX buffer ends 1 byte prior
		PTR_BASE bufferEnd;			// Last byte of RX buffer
		PTR_BASE txHead;			// Head pointer for TX
		PTR_BASE txTail;			// Tail pointer for TX
		PTR_BASE rxHead;			// Head pointer for RX
		PTR_BASE rxTail;			// Tail pointer for RX
		DWORD eventTime;			// Packet retransmissions, state changes
		WORD eventTime2;			// Window updates, automatic transmission
		union {
			WORD delayedACKTime;	// Delayed Acknowledgement timer
			WORD closeWaitTime;		// TCP_CLOSE_WAIT timeout timer
		} OverlappedTimers;
		TCP_STATE smState;			// State of this socket
		BYTE vUnackedKeepalives;	// Count of how many keepalives have been sent with no response
		SHORT Flags;                // flags see #defines above (unsigned int)
		WORD_VAL remoteHash;	    // Consists of remoteIP, remotePort, localPort for connected sockets.  It is a localPort number only for listening server sockets.
		#if defined(STACK_USE_SSL)
			PTR_BASE sslTxHead;		// Position of data being written in next SSL application record Also serves as cache of localSSLPort when smState = TCP_LISTENING
			PTR_BASE sslRxHead;		// Position of incoming data not yet handled by SSL
			BYTE sslStubID;			// Which sslStub is associated with this connection
			BYTE sslReqMessage;		// Currently requested SSL message
		#endif
	} TCB_STUB;
	// Remainder of TCP Control Block data.
	// The rest of the TCB is stored in Ethernet buffer RAM or elsewhere as defined by vMemoryMedium.
	// Current size is 41 (PIC18), 42 (PIC24/dsPIC), or 48 bytes (PIC32)
	
	#define bFINSent 7
	#define bSYNSent 6
	//#define bRemoteHostIsROM 5
	#define bRXNoneACKed1 4
	#define bRXNoneACKed2 3
	
	typedef struct {
		DWORD		retryInterval;			// How long to wait before retrying transmission
		DWORD		MySEQ;					// Local sequence number
		DWORD		RemoteSEQ;				// Remote sequence number
		PTR_BASE	txUnackedTail;			// TX tail pointer for data that is not yet acked
		WORD_VAL	remotePort;				// Remote port number
		WORD_VAL	localPort;				// Local port number
		WORD		remoteWindow;			// Remote window size
		WORD		wFutureDataSize;		// How much out-of-order data has been received
		union {
			NODE_INFO	niRemoteMACIP;		// 10 bytes for MAC and IP address
			DWORD		dwRemoteHost;		// RAM or ROM pointer to a hostname string (ex: "www.microchip.com")
		} remote;
		SHORT		sHoleSize;				// Size of the hole, or -1 for none exists.  (0 indicates hole has just been filled)
		BYTE flags;
		WORD		wRemoteMSS;				// Maximum Segment Size option advirtised by the remote node during initial handshaking
		#if defined(STACK_USE_SSL)
			WORD_VAL	localSSLPort;	    // Local SSL port number (for listening sockets)
		#endif
		BYTE		retryCount;				// Counter for transmission retries
		//BYTE		vSocketPurpose;			// Purpose of socket (as defined in TCPIPConfig.h)
	} TCB;
	
	// Information about a socket
	typedef struct{
		NODE_INFO remote;		// NODE_INFO structure for remote node
		WORD_VAL remotePort;	// Port number associated with remote node
	} SOCKET_INFO;
	
	/****************************************************************************
	  Section:
		Function Declarations
	  ***************************************************************************/
	
	void TCPInit(void);
	SOCKET_INFO TCPGetRemoteInfo(TCP_SOCKET hTCP);
	BOOL TCPWasReset(TCP_SOCKET hTCP);
	BOOL TCPIsConnected(TCP_SOCKET hTCP);
	void TCPDisconnect(TCP_SOCKET hTCP);
	void TCPClose(TCP_SOCKET hTCP);
	WORD TCPIsPutReady(TCP_SOCKET hTCP);
	BOOL TCPPut(TCP_SOCKET hTCP, BYTE byte);
	WORD TCPPutArray(TCP_SOCKET hTCP, BYTE* Data, WORD Len);
	BOOL TCPPutArrayBlock(TCP_SOCKET hTCP, BYTE * data, size_type len, const DWORD * maxtime);
	BOOL TCPPutStringBlock(TCP_SOCKET hTCP,BYTE * data, const DWORD * maxtime);
	BYTE* TCPPutString(TCP_SOCKET hTCP, BYTE* Data);
	WORD TCPIsGetReady(TCP_SOCKET hTCP);
	WORD TCPGetRxFIFOFree(TCP_SOCKET hTCP);
	BOOL TCPGet(TCP_SOCKET hTCP, BYTE* byte);
	WORD TCPGetArray(TCP_SOCKET hTCP, BYTE* buffer, WORD count);
	BYTE TCPPeek(TCP_SOCKET hTCP, WORD wStart);
	WORD TCPPeekArray(TCP_SOCKET hTCP, BYTE *vBuffer, WORD wLen, WORD wStart);
	WORD TCPFindEx(TCP_SOCKET hTCP, BYTE cFind, WORD wStart, WORD wSearchLen, BOOL bTextCompare);
	WORD TCPFindArrayEx(TCP_SOCKET hTCP, BYTE* cFindArray, WORD wLen, WORD wStart, WORD wSearchLen, BOOL bTextCompare);
	WORD TCPFindString(TCP_SOCKET hTCP, unsigned char * cFindString, WORD wStart, WORD wSearchLen, BOOL bTextCompare);
	WORD TCPFindString(TCP_SOCKET hTCP, unsigned char * cFindString, BOOL bTextCompare);
	#define TCPFind(a,b,c,d)					TCPFindEx(a,b,c,0,d)
	#define TCPFindArray(a,b,c,d,e)				TCPFindArrayEx(a,b,c,d,0,e)
	#define TCPListen(port)			TCPOpen(0, TCP_OPEN_SERVER, port, TCP_PURPOSE_DEFAULT)
	#define TCPConnect(remote,port)	TCPOpen((DWORD)remote, TCP_OPEN_NODE_INFO, port, TCP_PURPOSE_DEFAULT)
	void TCPDiscard(TCP_SOCKET hTCP);
	BOOL TCPProcess(NODE_INFO* remote, IP_ADDR* localIP, WORD len);
	void TCPTick(void);
	void TCPFlush(TCP_SOCKET hTCP);
	
	// Create a server socket and ignore dwRemoteHost.
	#define TCP_OPEN_SERVER		0
	#ifdef STACK_CLIENT_MODE
		#ifdef STACK_USE_DNS
			// Create a client socket and use dwRemoteHost as a RAM pointer to a hostname string.
			#define TCP_OPEN_RAM_HOST	1
			// Create a client socket and use dwRemoteHost as a ROM pointer to a hostname string.
			#define TCP_OPEN_ROM_HOST	2
		#else
			// Emit an undeclared identifier diagnostic if code tries to use TCP_OPEN_RAM_HOST while the DNS client module is not enabled. 
			#define TCP_OPEN_RAM_HOST	You_need_to_enable_STACK_USE_DNS_to_use_TCP_OPEN_RAM_HOST
			// Emit an undeclared identifier diagnostic if code tries to use TCP_OPEN_ROM_HOST while the DNS client module is not enabled. 
			#define TCP_OPEN_ROM_HOST	You_need_to_enable_STACK_USE_DNS_to_use_TCP_OPEN_ROM_HOST
		#endif
		// Create a client socket and use dwRemoteHost as a literal IP address.
		#define TCP_OPEN_IP_ADDRESS	3u
		// Create a client socket and use dwRemoteHost as a pointer to a NODE_INFO structure containing the exact remote IP address and MAC address to use.
		#define TCP_OPEN_NODE_INFO	4u
	#else
		// Emit an undeclared identifier diagnostic if code tries to use TCP_OPEN_RAM_HOST while STACK_CLIENT_MODE feature is not enabled. 
		#define TCP_OPEN_RAM_HOST	You_need_to_enable_STACK_CLIENT_MODE_to_use_TCP_OPEN_RAM_HOST
		// Emit an undeclared identifier diagnostic if code tries to use TCP_OPEN_ROM_HOST while STACK_CLIENT_MODE feature is not enabled. 
		#define TCP_OPEN_ROM_HOST	You_need_to_enable_STACK_CLIENT_MODE_to_use_TCP_OPEN_ROM_HOST
		// Emit an undeclared identifier diagnostic if code tries to use TCP_OPEN_IP_ADDRESS while STACK_CLIENT_MODE feature is not enabled. 
		#define TCP_OPEN_IP_ADDRESS	You_need_to_enable_STACK_CLIENT_MODE_to_use_TCP_OPEN_IP_ADDRESS
		// Emit an undeclared identifier diagnostic if code tries to use TCP_OPEN_NODE_INFO while STACK_CLIENT_MODE feature is not enabled. 
		#define TCP_OPEN_NODE_INFO	You_need_to_enable_STACK_CLIENT_MODE_to_use_TCP_OPEN_NODE_INFO
	#endif
	
	TCP_SOCKET TCPOpen(IP_ADDR *RemoteIP, BYTE * RemoteHost, BYTE vRemoteHostType, WORD wPort);
	WORD TCPGetTxFIFOFull(TCP_SOCKET hTCP);
	BOOL TCPAdjustFIFOSize(TCP_SOCKET hTCP, WORD wMinRXSize, WORD wMinTXSize, BYTE vFlags);
	
	#define TCPFindROMArray(a,b,c,d,e) 		TCPFindArray(a,(BYTE*)b,c,d,e)
	#define TCPFindROMArrayEx(a,b,c,d,e,f) 	TCPFindArrayEx(a,(BYTE*)b,c,d,e,f)
	#define TCPPutROMArray(a,b,c)			TCPPutArray(a,(BYTE*)b,c)
	#define TCPPutROMString(a,b)			TCPPutString(a,(BYTE*)b)
	// Alias to TCPIsGetReady provided for API completeness
	#define TCPGetRxFIFOFull(a)					TCPIsGetReady(a)
	// Alias to TCPIsPutReady provided for API completeness
	#define TCPGetTxFIFOFree(a) 				TCPIsPutReady(a)
	
	#ifdef STACK_USE_SSL
		BOOL TCPStartSSLClient(TCP_SOCKET hTCP, BYTE* host);
		BOOL TCPStartSSLClientEx(TCP_SOCKET hTCP, BYTE* host, void * buffer, BYTE suppDataType);
		BOOL TCPStartSSLServer(TCP_SOCKET hTCP);
		BOOL TCPAddSSLListener(TCP_SOCKET hTCP, WORD port);
		BOOL TCPRequestSSLMessage(TCP_SOCKET hTCP, BYTE msg);
		BOOL TCPSSLIsHandshaking(TCP_SOCKET hTCP);
		BOOL TCPIsSSL(TCP_SOCKET hTCP);
		void TCPSSLHandshakeComplete(TCP_SOCKET hTCP);
		void TCPSSLDecryptMAC(TCP_SOCKET hTCP, ARCFOUR_CTX* ctx, WORD len);
		void TCPSSLInPlaceMACEncrypt(TCP_SOCKET hTCP, ARCFOUR_CTX* ctx, BYTE* MACSecret, WORD len);
		void TCPSSLPutRecordHeader(TCP_SOCKET hTCP, BYTE* hdr, BOOL recDone);
		WORD TCPSSLGetPendingTxSize(TCP_SOCKET hTCP);
		void TCPSSLHandleIncoming(TCP_SOCKET hTCP);
	#endif
	
	
	//TCP/IP memory management functions
	void tcp_copy_to_sram(PTR_BASE ptrDest, PTR_BASE ptrSource, WORD wLength);
	void tcp_copy_from_sram(PTR_BASE ptrDest, PTR_BASE ptrSource, WORD wLength);
	void tcp_copy_ram_to_ram(PTR_BASE ptrDest,PTR_BASE PtrSource,WORD wLength);
	void tcp_copy_from_sram_to_eth(PTR_BASE ptrDest,PTR_BASE ptrSource, WORD wLength);
	void tcp_copy_from_eth_to_sram(PTR_BASE ptrDest, PTR_BASE ptrSource, WORD wLength);
	void tcp_copy_from_sram_to_eth(PTR_BASE ptrDest,PTR_BASE ptrSource, WORD wLength);
	void tcp_copy_from_eth_to_sram(PTR_BASE ptrDest, PTR_BASE ptrSource, WORD wLength);	
	
	//copy from pic to sram
	inline void tcp_copy_to_sram(PTR_BASE ptrDest, PTR_BASE ptrSource, WORD wLength){
		stack_copy_to_sram(ptrDest,ptrSource,wLength); //see stackconf.h, here you define your own ram copy functions
	}
	
	//copy from sram to pic
	inline void tcp_copy_from_sram(PTR_BASE ptrDest, PTR_BASE ptrSource, WORD wLength){
		stack_copy_from_sram(ptrDest,ptrSource,wLength); //see stackconf.h, here you define your own ram copy functions
	}
	
	inline void tcp_copy_ram_to_ram(PTR_BASE ptrDest,PTR_BASE PtrSource,WORD wLength){
		stack_copy_sram_to_sram(ptrDest,PtrSource,wLength);
	}
	
#endif