

/* ENC28J60 Ethernet controller */

#include "ENC28J60.h"

MAC_ADDR enc_MAC_addr; //the MAC address of the module

//registers in ENC28J60

//each register has address size of 5 bits, so we use the upper 2 (or 3) bits for bank selection
/*BSEL1:BSEL0: Bank Select bits in ENC_ECON1
11 = SPI accesses registers in Bank 3
10 = SPI accesses registers in Bank 2
01 = SPI accesses registers in Bank 1
00 = SPI accesses registers in Bank 0*/
#define ENC_ADDRESSMASK 0b11111
#define ENC_BANKMASK 0b11100000
//Bank 0   upper 3 bits 000
#define ENC_ERDPTL   0x00
#define ENC_ERDPTH   0x01
#define ENC_EWRPTL   0x02
#define ENC_EWRPTH   0x03
#define ENC_ETXSTL   0x04
#define ENC_ETXSTH   0x05
#define ENC_ETXNDL   0x06
#define ENC_ETXNDH   0x07
#define ENC_ERXSTL   0x08
#define ENC_ERXSTH   0x09
#define ENC_ERXNDL   0x0A
#define ENC_ERXNDH   0x0B
#define ENC_ERXRDPTL 0x0C
#define ENC_ERXRDPTH 0x0D
#define ENC_ERXWRPTL 0x0E
#define ENC_ERXWRPTH 0x0F
#define ENC_EDMASTL	 0x10
#define ENC_EDMASTH	 0x11
#define ENC_EDMANDL	 0x12
#define ENC_EDMANDH	 0x13
#define ENC_EDMADSTL 0x14
#define ENC_EDMADSTH 0x15
#define ENC_EDMACSL	 0x16
#define ENC_EDMACSH	 0x17
#define ENC_EIE	     0x1B
#define ENC_EIR	     0x1C
#define ENC_ESTAT    0x1D
#define ENC_ECON2    0x1E
#define ENC_ECON1    0x1F

//Bank 1    upper 3 bits 001
#define ENC_EHT0	0x20
#define ENC_EHT1	0x21
#define ENC_EHT2	0x22
#define ENC_EHT3	0x23
#define ENC_EHT4	0x24
#define ENC_EHT5	0x25
#define ENC_EHT6	0x26
#define ENC_EHT7	0x27
#define ENC_EPMM0	0x28
#define ENC_EPMM1	0x29
#define ENC_EPMM2	0x2A
#define ENC_EPMM3	0x2B
#define ENC_EPMM4	0x2C
#define ENC_EPMM5	0x2D
#define ENC_EPMM6	0x2E
#define ENC_EPMM7	0x2F
#define ENC_EPMCSL	0x30
#define ENC_EPMCSH	0x31
#define ENC_EPMOL	0x34
#define ENC_EPMOH	0x35
#define ENC_ERXFCON	0x38
#define ENC_EPKTCNT	0x39

//Bank 2    upper 3 bits 010
#define ENC_MACON1	 0x40
#define ENC_MACON3	 0x42
#define ENC_MACON4	 0x43
#define ENC_MABBIPG	 0x44
#define ENC_MAIPGL	 0x46
#define ENC_MAIPGH	 0x47
#define ENC_MACLCON1 0x48
#define ENC_MACLCON2 0x49
#define ENC_MAMXFLL	 0x4A
#define ENC_MAMXFLH	 0x4B
#define ENC_MICMD	 0x52
#define ENC_MIREGADR 0x54
#define ENC_MIWRL	 0x56
#define ENC_MIWRH	 0x57
#define ENC_MIRDL	 0x58
#define ENC_MIRDH	 0x59

//Bank 3    upper 3 bits 011
#define ENC_MAADR5	0x60
#define ENC_MAADR6	0x61
#define ENC_MAADR3	0x62
#define ENC_MAADR4	0x63
#define ENC_MAADR1	0x64
#define ENC_MAADR2	0x65
#define ENC_EBSTSD	0x66
#define ENC_EBSTCON	0x67
#define ENC_EBSTCSL	0x68
#define ENC_EBSTCSH	0x69
#define ENC_MISTAT	0x6A
#define ENC_EREVID	0x72
#define ENC_ECOCON	0x75
#define ENC_EFLOCON	0x77
#define ENC_EPAUSL	0x78
#define ENC_EPAUSH	0x79

//EIE register bits
#define ENC_INTIE 7
#define ENC_PKTIE 6
#define ENC_DMAIE 5
#define ENC_LINKIE 4
#define ENC_TXIE 3
#define ENC_TXERIE 1
#define ENC_RXERIE 0

//EIR register bits
#define ENC_PKTIF 6
#define ENC_DMAIF 5
#define ENC_LINKIF 4
#define ENC_TXIF 3
#define ENC_TXERIF 1
#define ENC_RXERIF 0

//ESTAT register bits
#define ENC_INT 7
#define ENC_BUFER 6
#define ENC_LATECOL 4
#define ENC_RXBUSY 2
#define ENC_TXABRT 1
#define ENC_CLKRDY 0

//ECON2 register bits
#define ENC_AUTOINC 7
#define ENC_PKTDEC 6
#define ENC_PWRSV 5
#define ENC_VRPS 3

//ECON1 register bits
#define ENC_TXRST 7
#define ENC_RXRST 6
#define ENC_DMAST 5
#define ENC_CSUMEN 4
#define ENC_TXRTS 3
#define ENC_RXEN 2
#define ENC_BSEL1 1
#define ENC_BSEL0 0

//ERXFCON register bits
#define ENC_UCEN 7
#define ENC_ANDOR 6
#define ENC_CRCEN 5
#define ENC_PMEN 4
#define ENC_MPEN 3
#define ENC_HTEN 2
#define ENC_MCEN 1
#define ENC_BCEN 0

//MACON1 register bits
#define ENC_TXPAUS 3
#define ENC_RXPAUS 2
#define ENC_PASSALL 1
#define ENC_MARXEN 0

//MACON3 register bits
#define ENC_PADCFG2 7
#define ENC_PADCFG1 6
#define ENC_PADCFG0 5
#define ENC_TXCRCEN 4
#define ENC_PHDREN 3
#define ENC_HFRMEN 2
#define ENC_FRMLNEN 1
#define ENC_FULDPX 0

//MACON4 register bits
#define ENC_DEFER 6
#define ENC_BPEN 5
#define ENC_NOBKOFF 4

//MICMD register bits
#define ENC_MIISCAN 1
#define ENC_MIIRD  0

//EBSTCON register bits
#define ENC_PSV2 7
#define ENC_PSV1 6
#define ENC_PSV0 5
#define ENC_PSEL 4
#define ENC_TMSEL1 3
#define ENC_TMSEL0 2
#define ENC_TME 1
#define ENC_BISTST 0

//MISTAT register bits
#define ENC_NVALID 2
#define ENC_SCAN 1
#define ENC_BUSY 0

//ECOCON register bits
#define ENC_COCON2 2
#define ENC_COCON1 1
#define ENC_COCON0 0

//EFLOCON register bits
#define ENC_FULDPXS 2
#define ENC_FCEN1 1
#define ENC_FCEN0 0

//TXSTATUS bits:
#define ENC_Zero 55,3
#define ENC_TransmitLateCollision 29
#define ENC_TransmitVLANTaggedFrame 51
#define ENC_Backpressure_Applied 50
#define ENC_Transmit_Pause_Control_Frame 49
#define ENC_Transmit_Control_Frame 48
//#define ENC_Total_Bytes_Transmitted_on_Wire 47-32
#define ENC_Transmit_Giant 30
//#define ENC_Transmit_Late_Collision 29
#define ENC_Transmit_Transmit_excessive_Collision 28
#define ENC_Transmit_Excessive_Defer 27
#define ENC_Transmit_Packet_Defer 26
#define ENC_Transmit_Broadcase 25
#define ENC_Transmit_Multicast 24
#define ENC_Transmit_Done 23
#define ENC_Transmit_Length_Out_of_Range 22
#define ENC_Transmit_Length_Check_error 21
#define ENC_Transmit_CRC_Error 20
//#define ENC_Transmit_Collision_Count 19-16
//#define ENC_Transmit_Byte_Count 15-0


//opcodes
#define ENC_READREG   0b00000000
#define ENC_READBUFF  0b00111010
#define ENC_WRITEREG  0b01000000
#define ENC_WRITEBUFF 0b01111010
#define ENC_SETBIT    0b10000000
#define ENC_CLEARBIT  0b10100000
#define ENC_SYSRESET  0b11111111

#define ENC_PHCON1	0x00
#define ENC_PHSTAT1	0x01
#define ENC_PHID1	0x02
#define ENC_PHID2	0x03
#define ENC_PHCON2	0x10
#define ENC_PHSTAT2	0x11
#define ENC_PHIE	0x12
#define ENC_PHIR	0x13
#define ENC_PHLCON	0x14
#define ENC_PDPXMD  8

#define ENC_HDLDIS 8
#define ENC_LLSTAT 2

#define ETHER_IP    (unsigned char)(0x00)
#define ETHER_ARP   (unsigned char)(0x06)


/*static const unsigned char memory_bank0=0x00;
static const unsigned char memory_bank1=0x20;
static const unsigned char memory_bank2=0x40;
static const unsigned char memory_bank3=0x80;
static unsigned char memory_bank=0; //current memory bank in use*/
// Internal MAC level variables and flags.
static unsigned int NextPacketLocation;
static unsigned int CurrentPacketLocation;
static bool WasDiscarded;
static unsigned char ENCRevID;
static bit DuplexState;

// Prototypes of functions intended for MAC layer use only.
/*static void BankSel(unsigned char Register);
static unsigned char ReadETHReg(unsigned char Address);
static unsigned char ReadMACReg(unsigned char Address);
static void WriteReg(unsigned char Address, unsigned char Data);
static void BFCReg(unsigned char Address,unsigned char Data);
static void BFSReg(unsigned char Address, unsigned char Data);
static void SendSystemReset(void);
static unsigned int ReadPHYReg(unsigned char Register);
void MACPut(unsigned char val);*/


static void SendSystemReset();
static unsigned char ReadETHReg(unsigned char Address);
static unsigned char ReadMACReg(unsigned char Address);
unsigned int ReadPHYReg(unsigned char Register);
static void WriteReg(unsigned char Address,unsigned char Data);
static void BFCReg(unsigned char Address,unsigned char Data);
static void BFSReg(unsigned char Address, unsigned char Data);
void WritePHYReg(unsigned char Register, unsigned int Data);
static void BankSel(unsigned char Register);
void SetCLKOUT(unsigned char NewConfig);
unsigned char GetCLKOUT();
void SetLEDConfig(unsigned int NewConfig);

//chip enable:
void enc_chip_enable();
void enc_chip_disable();

//local functions:
static bool getbitfromarray(unsigned char * array, unsigned char index);
static unsigned char getbitsfromarray (unsigned char * array, unsigned char index, unsigned char count);

inline void enc_chip_enable (){
	enc_cs = 0;
}

inline void enc_chip_disable(){
	enc_cs = 1;
}

inline void enc_chip_reset(){
	enc_rs = 0;
	delay_ms(1);
	enc_rs = 1;
}

//returns if bit index in the array has been set
//index is the index of the bit in the array you want to test
//each byte has 8 bits
static bool getbitfromarray(unsigned char * array, unsigned char index){
	return array[index/8] & (1<<(index%8));
}

//index = position of MSbit in array, count = nr of bits to count DOWN
static unsigned char getbitsfromarray(unsigned char * array, unsigned char index, unsigned char count){
	unsigned char res=0;
	for(unsigned char i=0;i<count;i++){
		if (getbitfromarray(array,index)) res++;
		index--;
		res<<=1;
	}
	return res;
}

//NOTE: All code in this module expects Bank 0 to be currently selected.  If code ever changes the bank, it must restore it to Bank 0 before returning.

/******************************************************************************
 * Macro:        	void SetLEDConfig(WORD NewConfig)
 *
 * PreCondition:    SPI bus must be initialized (done in MACInit()).
 *
 * Input:           NewConfig - xxx0: Pulse stretching disabled
 *								xxx2: Pulse stretch to 40ms (default)
 *								xxx6: Pulse stretch to 73ms
 *								xxxA: Pulse stretch to 139ms
 *
 *								xx1x: LEDB - TX
 *								xx2x: LEDB - RX (default)
 *								xx3x: LEDB - collisions
 *								xx4x: LEDB - link
 *								xx5x: LEDB - duplex
 *								xx7x: LEDB - TX and RX
 *								xx8x: LEDB - on
 *								xx9x: LEDB - off
 *								xxAx: LEDB - blink fast
 *								xxBx: LEDB - blink slow
 *								xxCx: LEDB - link and RX
 *								xxDx: LEDB - link and TX and RX
 *								xxEx: LEDB - duplex and collisions
 *
 *								x1xx: LEDA - TX
 *								x2xx: LEDA - RX
 *								x3xx: LEDA - collisions
 *								x4xx: LEDA - link (default)
 *								x5xx: LEDA - duplex
 *								x7xx: LEDA - TX and RX
 *								x8xx: LEDA - on
 *								x9xx: LEDA - off
 *								xAxx: LEDA - blink fast
 *								xBxx: LEDA - blink slow
 *								xCxx: LEDA - link and RX
 *								xDxx: LEDA - link and TX and RX
 *								xExx: LEDA - duplex and collisions
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Writes the value of NewConfig into the PHLCON PHY register.
 *					The LED pins will beginning outputting the new
 *					configuration immediately.
 *
 * Note:
 *****************************************************************************/
inline void SetLEDConfig(unsigned int NewConfig){
		WritePHYReg(ENC_PHLCON, NewConfig);
}

/******************************************************************************
 * Function:        void MACInit(void)
 * Overview:        MACInit sets up the PIC's SPI module and all the
 *                  registers in the ENC28J60 so that normal operation can
 *                  begin.
 *****************************************************************************/
void MACInit(){
    unsigned char  i;
	enc_tris_cs = 0;
	enc_tris_rs = 0;
	enc_chip_disable();
    // If the RESET pin is connected, take the chip out of reset
	enc_chip_reset();
	serial_send_string("ETH reset");
    // RESET the entire ENC28J60, clearing all registers
    // Also wait for CLKRDY to become set.
    // Bit 3 in ESTAT is an unimplemented bit.  If it reads out as '1' that
    // means the part is in RESET or there is something wrong with the SPI
    // connection.  This loop makes sure that we can communicate with the
    // ENC28J60 before proceeding.
    
    do {
        SendSystemReset();
        i = ReadETHReg(ENC_ESTAT);
        /*serial_send_string("ESTAT:");
        serial_send_hex(i);
        serial_send_string("\r\n");*/
    } while((i & 0x08) || (~i & (1<<ENC_CLKRDY)));
	//serial_send_string("OK\r\n");
    // Start up in Bank 0 and configure the receive buffer boundary pointers
    // and the buffer write protect pointer (receive buffer read pointer)
    WasDiscarded = true;
    NextPacketLocation = RXSTART;

    WriteReg(ENC_ERXSTL, LOW(RXSTART));
    WriteReg(ENC_ERXSTH, HIGH(RXSTART));
    WriteReg(ENC_ERXRDPTL, LOW(RXSTOP));    // Write low byte first
    WriteReg(ENC_ERXRDPTH, HIGH(RXSTOP));   // Write high byte last
    WriteReg(ENC_ERXNDL, LOW(RXSTOP));
    WriteReg(ENC_ERXNDH, HIGH(RXSTOP));
    WriteReg(ENC_ETXSTL, LOW(TXSTART));
    WriteReg(ENC_ETXSTH, HIGH(TXSTART));
	
	//******
	//serial_send_string("TXSTART:0x");
	//serial_send_hex(ReadETHReg(ENC_ETXSTH));
	//serial_send_hex(ReadETHReg(ENC_ETXSTL));
	//serial_send_string("\r\n");
    
    // Write a permanant per packet control byte of 0x00
    WriteReg(ENC_EWRPTL, LOW(TXSTART));
    WriteReg(ENC_EWRPTH, HIGH(TXSTART));
    MACPut(0x00);

    // Enter Bank 1 and configure Receive Filters
    // (No need to reconfigure - Unicast OR Broadcast with CRC checking is
    // acceptable)
    // Write ERXFCON_CRCEN only to ERXFCON to enter promiscuous mode

    // Promiscious mode example:
    //BankSel(ERXFCON);
    //WriteReg((BYTE)ERXFCON, ERXFCON_CRCEN);
	//********
	
	//serial_send_string("TEST BANK SELECT(1):");
	//BankSel(ENC_EHT0);
	//serial_send_hex(ReadETHReg(ENC_ECON1)&0b11);
	//serial_send_string("(2):");
	//BankSel(ENC_MACON1);
	//serial_send_hex(ReadETHReg(ENC_ECON1)&0b11);
	//serial_send_string("(3):");
	//BankSel(ENC_MAADR5);
	//serial_send_hex(ReadETHReg(ENC_ECON1)&0b11);
	//serial_send_string("(0):");
	//BankSel(ENC_ERDPTL);
	//serial_send_hex(ReadETHReg(ENC_ECON1)&0b11);
	//serial_send_string("\r\n");
	
	
    // Enter Bank 2 and configure the MAC
    BankSel(ENC_MACON1);

    // Enable the receive portion of the MAC
    WriteReg(ENC_MACON1, 1<< ENC_TXPAUS | 1<<ENC_RXPAUS | 1<<ENC_MARXEN);

    // Pad packets to 60 bytes, add CRC, and check Type/Length field.
#if defined(FULL_DUPLEX)
    WriteReg(ENC_MACON3, 1<<ENC_PADCFG0 | 1<<ENC_TXCRCEN | 1<<ENC_FRMLNEN | 1<<ENC_FULDPX);
    WriteReg(ENC_MABBIPG, 0x15);
#else
    WriteReg(ENC_MACON3, 1<<ENC_PADCFG0 | 1<<ENC_TXCRCEN | 1<<ENC_FRMLNEN);
    WriteReg(ENC_MABBIPG, 0x12);
#endif
    // Allow infinite deferals if the medium is continuously busy
    // (do not time out a transmission if the half duplex medium is
    // completely saturated with other people's data)
    WriteReg(ENC_MACON4, 1<<ENC_DEFER);

    // Late collisions occur beyond 63+8 bytes (8 bytes for preamble/start of frame delimiter)
    // 55 is all that is needed for IEEE 802.3, but ENC28J60 B5 errata for improper link pulse
    // collisions will occur less often with a larger number.
    WriteReg(ENC_MACLCON2, 63);

    // Set non-back-to-back inter-packet gap to 9.6us.  The back-to-back
    // inter-packet gap (MABBIPG) is set by MACSetDuplex() which is called
    // later.
    WriteReg(ENC_MAIPGL, 0x12);
    WriteReg(ENC_MAIPGH, 0x0C);

    // Set the maximum packet size which the controller will accept
    WriteReg(ENC_MAMXFLL, LOW(6+6+2+1500+4));  // 1518 is the IEEE 802.3 specified limit
    WriteReg(ENC_MAMXFLH, HIGH(6+6+2+1500+4)); // 1518 is the IEEE 802.3 specified limit

    // Enter Bank 3 and initialize physical MAC address registers
    BankSel(ENC_MAADR1);
    WriteReg(ENC_MAADR1, enc_MAC_addr.v[0]);
    WriteReg(ENC_MAADR2, enc_MAC_addr.v[1]);
    WriteReg(ENC_MAADR3, enc_MAC_addr.v[2]);
    WriteReg(ENC_MAADR4, enc_MAC_addr.v[3]);
    WriteReg(ENC_MAADR5, enc_MAC_addr.v[4]);
    WriteReg(ENC_MAADR6, enc_MAC_addr.v[5]);

    // Disable the CLKOUT output to reduce EMI generation
    WriteReg(ENC_ECOCON, 0x00);   // Output off (0V)
    //WriteReg((BYTE)ECOCON, 0x01); // 25.000MHz
    //WriteReg((BYTE)ECOCON, 0x03); // 8.3333MHz (*4 with PLL is 33.3333MHz)

    // Get the Rev ID so that we can implement the correct errata workarounds
    ENCRevID = ReadETHReg(ENC_EREVID);

    // Disable half duplex loopback in PHY.  Bank bits changed to Bank 2 as a
    // side effect.
    WritePHYReg(ENC_PHCON2, 1<<ENC_HDLDIS);
    
    //serial_send_string("TEST READ PHYREG:");
    //unsigned int res = ReadPHYReg(ENC_PHCON2);
    //serial_send_hex(HIGH(res));
    //serial_send_hex(LOW(res));
    //serial_send_string("\r\n");
    
    
    // Configure LEDA to display LINK status, LEDB to display TX/RX activity
    SetLEDConfig(0x3472);

    // Set the MAC and PHY into the proper duplex state
#if defined(FULL_DUPLEX)
    WritePHYReg(ENC_PHCON1, 1<<ENC_PDPXMD);
#elif defined(HALF_DUPLEX)
    WritePHYReg(ENC_PHCON1, 0x0000);
#else
    // Use the external LEDB polarity to determine weather full or half duplex
    // communication mode should be set.
    {
        unsigned char Register;
        unsigned int PhyReg;

        // Read the PHY duplex mode
        PhyReg = ReadPHYReg(ENC_PHCON1);
        DuplexState = PhyReg.ENC_PDPXMD;// PhyReg.PHCON1bits.;

        // Set the MAC to the proper duplex mode
        BankSel(ENC_MACON3);
        Register = ReadMACReg(ENC_MACON3);
        Register.ENC_FULDPX = PhyReg.ENC_PDPXMD;
        WriteReg(ENC_MACON3, Register);
        // Set the back-to-back inter-packet gap time to IEEE specified
        // requirements.  The meaning of the MABBIPG value changes with the duplex
        // state, so it must be updated in this function.
        // In full duplex, 0x15 represents 9.6us; 0x12 is 9.6us in half duplex
        WriteReg(ENC_MABBIPG, PhyReg.ENC_PDPXMD ? 0x15 : 0x12);
    }
#endif

    BankSel(ENC_ERDPTL);        // Return to default Bank 0
    // Enable packet reception
    BFSReg(ENC_ECON1, 1<<ENC_RXEN);
}//end MACInit


/******************************************************************************
 * Function:        BOOL MACIsLinked(void)
 * Output:          TRUE: If the PHY reports that a link partner is present
 *                        and the link has been up continuously since the last
 *                        call to MACIsLinked()
 *                  FALSE: If the PHY reports no link partner, or the link went
 *                         down momentarily since the last call to MACIsLinked()
 * Overview:        Returns the PHSTAT1.LLSTAT bit.
 *****************************************************************************/
bool MACIsLinked(){
    // LLSTAT is a latching low link status bit.  Therefore, if the link
    // goes down and comes back up before a higher level stack program calls
    // MACIsLinked(), MACIsLinked() will still return FALSE.  The next
    // call to MACIsLinked() will return TRUE (unless the link goes down
    // again).
    //pushcs(0x11);
    bool res= (ReadPHYReg(ENC_PHSTAT1) & (((unsigned int)1) << ENC_LLSTAT))!=0;
    //popcs();
    return res;
}

/******************************************************************************
 * Function:        BOOL MACIsTxReady(void)
 * Output:          TRUE: If no Ethernet transmission is in progress
 *                  FALSE: If a previous transmission was started, and it has
 *                         not completed yet.  While FALSE, the data in the
 *                         transmit buffer and the TXST/TXND pointers must not
 *                         be changed.
 * Overview:        Returns the ECON1.TXRTS bit
 *****************************************************************************/
bool MACIsTxReady(){
    return !(ReadETHReg(ENC_ECON1) & (1<<ENC_TXRTS)!=0);
}

/******************************************************************************
 * Function:        void MACDiscardRx(void)
 * Overview:        Marks the last received packet (obtained using
 *                  MACGetHeader())as being processed and frees the buffer
 *                  memory associated with it
 * Note:            Is is safe to call this function multiple times between
 *                  MACGetHeader() calls.  Extra packets won't be thrown away
 *                  until MACGetHeader() makes it available.
 *****************************************************************************/
void MACDiscardRx(void){
    WORD_VAL NewRXRDLocation;

    // Make sure the current packet was not already discarded
    if(WasDiscarded) return;
    WasDiscarded = true;

    // Decrement the next packet pointer before writing it into
    // the ERXRDPT registers.  This is a silicon errata workaround.
    // RX buffer wrapping must be taken into account if the
    // NextPacketLocation is precisely RXSTART.
    NewRXRDLocation.Val = NextPacketLocation - 1;
    if(NewRXRDLocation.Val > RXSTOP){
        NewRXRDLocation.Val = RXSTOP;
    }

    // Decrement the RX packet counter register, EPKTCNT
    BFSReg(ENC_ECON2, 1<<ENC_PKTDEC);

    // Move the receive read pointer to unwrite-protect the memory used by the
    // last packet.  The writing order is important: set the low byte first,
    // high byte last.
    WriteReg(ENC_ERXRDPTL, NewRXRDLocation.v[0]);
    WriteReg(ENC_ERXRDPTH, NewRXRDLocation.v[1]);
}

/******************************************************************************
 * Function:        WORD MACGetFreeRxSize(void)
 * Output:          A WORD estimate of how much RX buffer space is free at
 *                  the present time.
 *****************************************************************************/
unsigned int MACGetFreeRxSize(){
    WORD_VAL ReadPT, WritePT;
    // Read the Ethernet hardware buffer write pointer.  Because packets can be
    // received at any time, it can change between reading the low and high
    // bytes.  A loop is necessary to make certain a proper low/high byte pair
    // is read.
    BankSel(ENC_EPKTCNT);
    //pushcs(0x06);
    do {
        // Save EPKTCNT in a temporary location
        ReadPT.v[0] = ReadETHReg(ENC_EPKTCNT);

        BankSel(ENC_ERXWRPTL);
        WritePT.v[0] = ReadETHReg(ENC_ERXWRPTL);
        WritePT.v[1] = ReadETHReg(ENC_ERXWRPTH);

        BankSel(ENC_EPKTCNT);
    } while(ReadETHReg(ENC_EPKTCNT) != ReadPT.v[0]);
	//popcs();
    // Determine where the write protection pointer is
    BankSel(ENC_ERXRDPTL);
    ReadPT.v[0] = ReadETHReg(ENC_ERXRDPTL);
    ReadPT.v[1] = ReadETHReg(ENC_ERXRDPTH);

    // Calculate the difference between the pointers, taking care to account
    // for buffer wrapping conditions
    if(WritePT.Val > ReadPT.Val){
        return (RXSTOP - RXSTART) - (WritePT.Val - ReadPT.Val);
    }else if(WritePT.Val == ReadPT.Val){
        return RXSIZE - 1;
    }else{
        return ReadPT.Val - WritePT.Val;
    }
}


/******************************************************************************
 * Function:        BOOL MACGetHeader(MAC_ADDR *remote, BYTE* type)
 * Input:           *remote: Location to store the Source MAC address of the
 *                           received frame.
 *                  *type: Location of a BYTE to store the constant
 *                         MAC_UNKNOWN, ETHER_IP, or ETHER_ARP, representing
 *                         the contents of the Ethernet type field.
 * Output:          TRUE: If a packet was waiting in the RX buffer.  The
 *                        remote, and type values are updated.
 *                  FALSE: If a packet was not pending.  remote and type are
 *                         not changed.
 * Side Effects:    Last packet is discarded if MACDiscardRx() hasn't already
 *                  been called.
 *****************************************************************************/
bool MACGetHeader(MAC_ADDR *remote, unsigned char * type){
    ENC_PREAMBLE header;
    unsigned char PacketCount;

    // Test if at least one packet has been received and is waiting
    BankSel(ENC_EPKTCNT);
    PacketCount = ReadETHReg(ENC_EPKTCNT);
    BankSel(ENC_ERDPTL);
    if(PacketCount == 0)
        return false;

    // Make absolutely certain that any previous packet was discarded
    if(WasDiscarded == false){
        MACDiscardRx();
        return false;
    }

    // Set the SPI read pointer to the beginning of the next unprocessed packet
    CurrentPacketLocation = NextPacketLocation;
    WriteReg(ENC_ERDPTL, ((WORD_VAL)CurrentPacketLocation).v[0]);
    WriteReg(ENC_ERDPTH, ((WORD_VAL)CurrentPacketLocation).v[1]);

    // Obtain the MAC header from the Ethernet buffer
    MACGetArray((unsigned char*)&header, (unsigned int) sizeof(ENC_PREAMBLE));

    // The EtherType field, like most items transmitted on the Ethernet medium
    // are in big endian.
    header.Type = swaps(header.Type);

    // Validate the data returned from the ENC28J60.  Random data corruption,
    // such as if a single SPI bit error occurs while communicating or a
    // momentary power glitch could cause this to occur in rare circumstances.
    /*if(header.NextPacketPointer > RXSTOP || (header.NextPacketPointer & 1)!=0 ||
       header.StatusVector[6] & 0b11110000 ||  //zero bits
	   header.StatusVector[2] & 0b10000 || //CRC error
	   createshort(header.StatusVector[0],header.StatusVector[1]) > 1518 || //ByteCount
	   ! 
       header.StatusVector.bits.CRCError ||
       header.StatusVector.bits.ByteCount > 1518u ||
       !header.StatusVector.bits.ReceiveOk){
        Reset();
    }*/
    /*if(header.NextPacketPointer > RXSTOP || (header.NextPacketPointer & 1)!=0 ||
	   (header.StatusVector[3] & 0b10000000)!=0 || //zero bit
	   (header.StatusVector[2] & 0b10000)!=0 || //CRC error
	   createshort(header.StatusVector[0],header.StatusVector[1]) > 1518 || //ByteCount
	   !(header.StatusVector[2] & 0b10000000))
	{
		enc_error(1);
	}*/
	if(header.NextPacketPointer > RXSTOP){ enc_error(1); return false;}
	if((header.NextPacketPointer & 1)!=0){ enc_error(2); return false;}
	if((header.StatusVector[3] & 0b10000000)!=0){ enc_error(3); return false;}
	if((header.StatusVector[2] & 0b10000)!=0){ enc_error(4); return false;}
	if(createshort(header.StatusVector[0],header.StatusVector[1]) > 1518){ enc_error(5); return false;}
	if(!(header.StatusVector[2] & 0b10000000)){ enc_error(6); return false;}
	

    // Save the location where the hardware will write the next packet to
    NextPacketLocation = header.NextPacketPointer;

    // Return the Ethernet frame's Source MAC address field to the caller
    // This parameter is useful for replying to requests without requiring an
    // ARP cycle.
    memcpy(remote->v, header.SourceMACAddr.v, sizeof(MAC_ADDR));

    // Return a simplified version of the EtherType field to the caller
    *type = MAC_UNKNOWN;
    if( ((HIGH(header.Type) == 0x08) && ((LOW(header.Type) == ETHER_IP) || (LOW(header.Type) == ETHER_ARP)) )){
        *type = LOW(header.Type);
    }

    // Mark this packet as discardable
    WasDiscarded = false;
    return true;
}

/******************************************************************************
 * Function:        void MACPutHeader(MAC_ADDR *remote, BYTE type, WORD dataLen)
 * PreCondition:    MACIsTxReady() must return TRUE.
 * Input:           *remote: Pointer to memory which contains the destination
 *                           MAC address (6 bytes)
 *                  type: The constant ETHER_ARP or ETHER_IP, defining which
 *                        value to write into the Ethernet header's type field.
 *                  dataLen: Length of the Ethernet data payload
 * Note:            Because of the dataLen parameter, it is probably
 *                  advantagous to call this function immediately before
 *                  transmitting a packet rather than initially when the
 *                  packet is first created.  The order in which the packet
 *                  is constructed (header first or data first) is not
 *                  important.
 *****************************************************************************/
void MACPutHeader(MAC_ADDR *remote, unsigned char type, unsigned int dataLen){
    // Set the SPI write pointer to the beginning of the transmit buffer (post per packet control byte)
    unsigned int start =TXSTART+(unsigned int)(1);
    WriteReg(ENC_EWRPTL, ((WORD_VAL)start).v[0]);
    WriteReg(ENC_EWRPTH, ((WORD_VAL)start).v[1]);

    // Calculate where to put the TXND pointer
    dataLen += (unsigned int)sizeof(ETHER_HEADER) + TXSTART;

    // Write the TXND pointer into the registers, given the dataLen given
    WriteReg(ENC_ETXNDL, ((WORD_VAL)dataLen).v[0]);
    WriteReg(ENC_ETXNDH, ((WORD_VAL)dataLen).v[1]);

    // Set the per-packet control byte and write the Ethernet destination
    // address
    MACPutArray((unsigned char*)remote, sizeof(MAC_ADDR));

    // Write our MAC address in the Ethernet source field
    MACPutArray((unsigned char*)&enc_MAC_addr, sizeof(enc_MAC_addr));

    // Write the appropriate Ethernet Type WORD for the protocol being used
    MACPut(0x08);
    MACPut((type == MAC_IP) ? ETHER_IP : ETHER_ARP);
}

/******************************************************************************
 * Function:        void MACFlush(void)
 * PreCondition:    A packet has been created by calling MACPut() and
 *                  MACPutHeader().
 * Overview:        MACFlush causes the current TX packet to be sent out on
 *                  the Ethernet medium.  The hardware MAC will take control
 *                  and handle CRC generation, collision retransmission and
 *                  other details.
 * Note:            After transmission completes (MACIsTxReady() returns TRUE),
 *                  the packet can be modified and transmitted again by calling
 *                  MACFlush() again.  Until MACPutHeader() or MACPut() is
 *                  called (in the TX data area), the data in the TX buffer
 *                  will not be corrupted.
 *****************************************************************************/
void MACFlush(void){
    // Reset transmit logic if a TX Error has previously occured
    // This is a silicon errata workaround
    BFSReg(ENC_ECON1, 1 << ENC_TXRST);
    BFCReg(ENC_ECON1, 1 << ENC_TXRST);
    BFCReg(ENC_EIR, 1 << ENC_TXERIF | 1 << ENC_TXIF);

    // Start the transmission
    // After transmission completes (MACIsTxReady() returns TRUE), the packet
    // can be modified and transmitted again by calling MACFlush() again.
    // Until MACPutHeader() is called, the data in the TX buffer will not be
    // corrupted.
    BFSReg(ENC_ECON1, 1 << ENC_TXRTS);

    // Revision B5 and B7 silicon errata workaround
    if(ENCRevID == 0x05u || ENCRevID == 0x06u){
        unsigned int AttemptCounter = 0x0000;
        //pushcs(0x07);
        while(!(ReadETHReg(ENC_EIR) & (1<<ENC_TXERIF | 1<<ENC_TXIF)) && (++AttemptCounter < 1000));
        //popcs();
        if(ReadETHReg(ENC_EIR)& 1<< ENC_TXERIF || (AttemptCounter >= 1000)){
            WORD_VAL ReadPtrSave;
            WORD_VAL TXEnd;
            unsigned char TXStatus[7];
            unsigned char i;

            // Cancel the previous transmission if it has become stuck set
            BFCReg(ENC_ECON1, 1<<ENC_TXRTS);

            // Save the current read pointer (controlled by application)
            ReadPtrSave.v[0] = ReadETHReg(ENC_ERDPTL);
            ReadPtrSave.v[1] = ReadETHReg(ENC_ERDPTH);

            // Get the location of the transmit status vector
            TXEnd.v[0] = ReadETHReg(ENC_ETXNDL);
            TXEnd.v[1] = ReadETHReg(ENC_ETXNDH);
            ((unsigned int)TXEnd)++;

            // Read the transmit status vector
            WriteReg(ENC_ERDPTL, TXEnd.v[0]);
            WriteReg(ENC_ERDPTH, TXEnd.v[1]);
            MACGetArray((unsigned char*)&TXStatus, sizeof(TXStatus));

            // Implement retransmission if a late collision occured (this can
            // happen on B5 when certain link pulses arrive at the same time
            // as the transmission)
            for(i = 0; i < 16; i++){
                if((ReadETHReg(ENC_EIR) & 1 << ENC_TXERIF) && getbitfromarray(TXStatus,ENC_TransmitLateCollision)){
                    // Reset the TX logic
                    BFSReg(ENC_ECON1, 1<<ENC_TXRST);
                    BFCReg(ENC_ECON1, 1<<ENC_TXRST);
                    BFCReg(ENC_EIR, 1<<ENC_TXERIF | 1<<ENC_TXIF);

                    // Transmit the packet again
                    BFSReg(ENC_ECON1, 1<<ENC_TXRTS);
                    //pushcs(0x08);
                    while(!(ReadETHReg(ENC_EIR) & (1<<ENC_TXERIF | 1<<ENC_TXIF)));
					//popcs();
                    // Cancel the previous transmission if it has become stuck set
                    BFCReg(ENC_ECON1, 1<<ENC_TXRTS);

                    // Read transmit status vector
                    WriteReg(ENC_ERDPTL, TXEnd.v[0]);
                    WriteReg(ENC_ERDPTH, TXEnd.v[1]);
                    MACGetArray(TXStatus, sizeof(TXStatus));
                }else{
                    break;
                }
            }
            // Restore the current read pointer
            WriteReg(ENC_ERDPTL, ReadPtrSave.v[0]);
            WriteReg(ENC_ERDPTH, ReadPtrSave.v[1]);
        }
    }
}

/******************************************************************************
 * Function:        void MACSetReadPtrInRx(WORD offset)
 * PreCondition:    A packet has been obtained by calling MACGetHeader() and
 *                  getting a TRUE result.
 * Input:           offset: WORD specifying how many bytes beyond the Ethernet
 *                          header's type field to relocate the SPI read
 *                          pointer.
 * Overview:        SPI read pointer are updated.  All calls to
 *                  MACGet() and MACGetArray() will use these new values.
 * Note:            RXSTOP must be statically defined as being > RXSTART for
 *                  this function to work correctly.  In other words, do not
 *                  define an RX buffer which spans the 0x1FFF->0x0000 memory
 *                  boundary.
 *****************************************************************************/
void MACSetReadPtrInRx(unsigned int offset){
    unsigned int ReadPT;
    // Determine the address of the beginning of the entire packet
    // and adjust the address to the desired location
    ReadPT = CurrentPacketLocation + sizeof(ENC_PREAMBLE) + offset;

    // Since the receive buffer is circular, adjust if a wraparound is needed
    if(ReadPT > RXSTOP) ReadPT -= RXSIZE;

    // Set the SPI read pointer to the new calculated value
    WriteReg(ENC_ERDPTL, ((WORD_VAL)ReadPT).v[0]);
    WriteReg(ENC_ERDPTH, ((WORD_VAL)ReadPT).v[1]);
}

/******************************************************************************
 * Function:        PTR_BASE MACSetWritePtr(PTR_BASE Address)
 * Input:           Address: Address to seek to
 * Output:          WORD: Old EWRPT location
 * Overview:        SPI write pointer is updated.  All calls to
 *                  MACPut() and MACPutArray() will use this new value.
 * Note:            None
 *****************************************************************************/
unsigned int MACSetWritePtr(unsigned int address){
    WORD_VAL oldVal;
    oldVal.v[0] = ReadETHReg(ENC_EWRPTL);
    oldVal.v[1] = ReadETHReg(ENC_EWRPTH);
    // Set the SPI write pointer to the new calculated value
    WriteReg(ENC_EWRPTL, ((WORD_VAL)address).v[0]);
    WriteReg(ENC_EWRPTH, ((WORD_VAL)address).v[1]);

    return oldVal.Val;
}

/******************************************************************************
 * Function:        PTR_BASE MACSetReadPtr(PTR_BASE Address)
 * Input:           Address: Address to seek to
 * Output:          WORD: Old ERDPT value
 * Overview:        SPI write pointer is updated.  All calls to
 *                  MACPut() and MACPutArray() will use this new value.
 *****************************************************************************/
unsigned int MACSetReadPtr(unsigned int address){
    WORD_VAL oldVal;

    oldVal.v[0] = ReadETHReg(ENC_ERDPTL);
    oldVal.v[1] = ReadETHReg(ENC_ERDPTH);

    // Set the SPI write pointer to the new calculated value
    WriteReg(ENC_ERDPTL, ((WORD_VAL)address).v[0]);
    WriteReg(ENC_ERDPTH, ((WORD_VAL)address).v[1]);

    return (unsigned int) oldVal;
}

/******************************************************************************
 * Function:        WORD MACCalcRxChecksum(WORD offset, WORD len)
 * Input:           offset  - Number of bytes beyond the beginning of the
 *                          Ethernet data (first byte after the type field)
 *                          where the checksum should begin
 *                  len     - Total number of bytes to include in the checksum
 * Output:          16-bit checksum as defined by RFC 793.
 * Overview:        This function performs a checksum calculation in the MAC
 *                  buffer itself
 *****************************************************************************/
unsigned int MACCalcRxChecksum(unsigned int offset, unsigned int len){
    WORD_VAL temp;
    WORD_VAL RDSave;
    // Add the offset requested by firmware plus the Ethernet header
    temp.Val = CurrentPacketLocation + sizeof(ENC_PREAMBLE) + offset;
    if(temp.Val > RXSTOP){       // Adjust value if a wrap is needed
        temp.Val -= RXSIZE;
    }

    RDSave.v[0] = ReadETHReg(ENC_ERDPTL);
    RDSave.v[1] = ReadETHReg(ENC_ERDPTH);

    WriteReg(ENC_ERDPTL, temp.v[0]);
    WriteReg(ENC_ERDPTH, temp.v[1]);

    temp.Val = CalcIPBufferChecksum(len);

    WriteReg(ENC_ERDPTL, RDSave.v[0]);
    WriteReg(ENC_ERDPTH, RDSave.v[1]);

    return temp.Val;
    
    /*unsigned int temp;
    WORD_VAL RDSave;
    // Add the offset requested by firmware plus the Ethernet header
    temp = CurrentPacketLocation + sizeof(ENC_PREAMBLE) + offset;
    if(temp > RXSTOP){       // Adjust value if a wrap is needed
        temp -= RXSIZE;
    }

    RDSave.v[0] = ReadETHReg(ENC_ERDPTL);
    RDSave.v[1] = ReadETHReg(ENC_ERDPTH);

    WriteReg(ENC_ERDPTL, ((WORD_VAL)temp).v[0]);
    WriteReg(ENC_ERDPTH, ((WORD_VAL)temp).v[1]);

    temp = CalcIPBufferChecksum(len);

    WriteReg(ENC_ERDPTL, RDSave.v[0]);
    WriteReg(ENC_ERDPTH, RDSave.v[1]);

    return temp;*/
}

/******************************************************************************
 * Function:        WORD CalcIPBufferChecksum(WORD len)
 * PreCondition:    Read buffer pointer set to starting of checksum data
 * Input:           len: Total number of bytes to calculate the checksum over.
 *                       The first byte included in the checksum is the byte
 *                       pointed to by ERDPT, which is updated by calls to
 *                       MACSetReadPtr(), MACGet(), MACGetArray(),
 *                       MACGetHeader(), etc.
 * Output:          16-bit checksum as defined by RFC 793
 * Side Effects:    None
 * Overview:        This function performs a checksum calculation in the MAC
 *                  buffer itself.  The ENC28J60 has a hardware DMA module
 *                  which can calculate the checksum faster than software, so
 *                  this function replaces the CaclIPBufferChecksum() function
 *                  defined in the helpers.c file.  Through the use of
 *                  preprocessor defines, this replacement is automatic.
 * Note:            This function works either in the RX buffer area or the TX
 *                  buffer area.  No validation is done on the len parameter.
 *****************************************************************************/
unsigned int CalcIPBufferChecksum(unsigned int len){
    WORD_VAL Start;
    unsigned long Checksum = {0x00000000};
    unsigned int ChunkLen;
    unsigned char DataBuffer[20];
    unsigned int *DataPtr;

    // Save the SPI read pointer starting address
    Start.v[0] = ReadETHReg(ENC_ERDPTL);
    Start.v[1] = ReadETHReg(ENC_ERDPTH);
	//pushcs(0x09);
    while(len){
        // Obtain a chunk of data (less SPI overhead compared
        // to requesting one byte at a time)
        ChunkLen = len > sizeof(DataBuffer) ? sizeof(DataBuffer) : len;
        MACGetArray(DataBuffer, ChunkLen);

        len -= ChunkLen;

        // Take care of a last odd numbered data byte
        if((unsigned char)ChunkLen & 1){
            DataBuffer[ChunkLen] = 0x00;
            ChunkLen++;
        }

        // Calculate the checksum over this chunk
        DataPtr = (unsigned int*)DataBuffer;
        //pushcs(0x0A);
        while(ChunkLen){
            Checksum += *DataPtr++;
            ChunkLen -= 2;
        }
        //popcs();
    }
	//popcs();
    // Restore old read pointer location
    WriteReg(ENC_ERDPTL, Start.v[0]);
    WriteReg(ENC_ERDPTH, Start.v[1]);

    // Do an end-around carry (one's complement arrithmatic)
    Checksum = (unsigned long)((DWORD_VAL)Checksum).w[0] + (unsigned long)((DWORD_VAL)Checksum).w[1];

    // Do another end-around carry in case if the prior add
    // caused a carry out
    ((DWORD_VAL)Checksum).w[0] += ((DWORD_VAL)Checksum).w[1];

    // Return the resulting checksum
    return ~((DWORD_VAL)Checksum).w[0];
}


/******************************************************************************
 * Function:        void MACMemCopyAsync(PTR_BASE destAddr, PTR_BASE sourceAddr, WORD len)
 * Input:           destAddr:   Destination address in the Ethernet memory to
 *                              copy to.  If (PTR_BASE)-1 is specified, the 
 *								current EWRPT value will be used instead.
 *                  sourceAddr: Source address to read from.  If (PTR_BASE)-1 is
 *                              specified, the current ERDPT value will be used
 *                              instead.
 *                  len:        Number of bytes to copy
 * Side Effects:    None
 * Overview:        Bytes are asynchrnously transfered within the buffer.  Call
 *                  MACIsMemCopyDone() to see when the transfer is complete.
 * Note:            If a prior transfer is already in progress prior to
 *                  calling this function, this function will block until it
 *                  can start this transfer.
 *                  If (PTR_BASE)-1 is used for the sourceAddr or destAddr
 *                  parameters, then that pointer will get updated with the
 *                  next address after the read or write.
 *****************************************************************************/
 //this function is still not working properly sometimes source address is not set correctly (ping packets get corrupted)
void MACMemCopyAsync(unsigned int destAddr, unsigned int sourceAddr, unsigned int len){
    WORD_VAL ReadSave, WriteSave;
    bool UpdateWritePointer = false;
    bool UpdateReadPointer = false;
	unsigned char h,l;

    if(destAddr == (unsigned int)-1){
        UpdateWritePointer = true;
        destAddr = createshort(ReadETHReg(ENC_EWRPTL),ReadETHReg(ENC_EWRPTH));
    }
    if(sourceAddr == (unsigned int)-1){
        UpdateReadPointer = true;
        l = ReadETHReg(ENC_ERDPTL);
        h = ReadETHReg(ENC_ERDPTH);
        sourceAddr = createshort(l,h);
    }

    // Handle special conditions where len == 0 or len == 1
    // The DMA module is not capable of handling those corner cases
    if(len <= 1){
        if(!UpdateReadPointer){
            ReadSave.v[0] = ReadETHReg(ENC_ERDPTL);
            ReadSave.v[1] = ReadETHReg(ENC_ERDPTH);
        }
        if(!UpdateWritePointer){
            WriteSave.v[0] = ReadETHReg(ENC_EWRPTL);
            WriteSave.v[1] = ReadETHReg(ENC_EWRPTH);
        }
        WriteReg(ENC_ERDPTL, ((WORD_VAL)sourceAddr).v[0]);
        WriteReg(ENC_ERDPTH, ((WORD_VAL)sourceAddr).v[1]);
        WriteReg(ENC_EWRPTL, ((WORD_VAL)destAddr).v[0]);
        WriteReg(ENC_EWRPTH, ((WORD_VAL)destAddr).v[1]);
        while(len--)
            MACPut(MACGet());
        if(!UpdateReadPointer){
            WriteReg(ENC_ERDPTL, ReadSave.v[0]);
            WriteReg(ENC_ERDPTH, ReadSave.v[1]);
        }
        if(!UpdateWritePointer){
            WriteReg(ENC_EWRPTL, WriteSave.v[0]);
            WriteReg(ENC_EWRPTH, WriteSave.v[1]);
        }
    }else{
        if(UpdateWritePointer){
            WriteSave.Val = destAddr + len;
            WriteReg(ENC_EWRPTL, WriteSave.v[0]);
            WriteReg(ENC_EWRPTH, WriteSave.v[1]);
        }
        len += sourceAddr - 1;
        while(ReadETHReg(ENC_ECON1) & (1 << ENC_DMAST));
        WriteReg(ENC_EDMASTL, ((WORD_VAL)sourceAddr).v[0]);
        WriteReg(ENC_EDMASTH, ((WORD_VAL)sourceAddr).v[1]);
        WriteReg(ENC_EDMADSTL, ((WORD_VAL)destAddr).v[0]);
        WriteReg(ENC_EDMADSTH, ((WORD_VAL)destAddr).v[1]);
        if((sourceAddr <= RXSTOP) && (len > RXSTOP)) //&& (sourceAddr >= RXSTART))
            len -= RXSIZE;
        WriteReg(ENC_EDMANDL, ((WORD_VAL)len).v[0]);
        WriteReg(ENC_EDMANDH, ((WORD_VAL)len).v[1]);
        
        BFCReg(ENC_ECON1, 1<<ENC_CSUMEN);
        BFSReg(ENC_ECON1, 1<<ENC_DMAST);
        if(UpdateReadPointer){
            len++;
            if((sourceAddr <= RXSTOP) && (len > RXSTOP)) //&& (sourceAddr >= RXSTART))
                len -= RXSIZE;
            WriteReg(ENC_ERDPTL, ((WORD_VAL)len).v[0]);
            WriteReg(ENC_ERDPTH, ((WORD_VAL)len).v[1]);
        }
    }
}

bool MACIsMemCopyDone(void){
    return !(ReadETHReg(ENC_ECON1) & (1 << ENC_DMAST));
}

/******************************************************************************
 * Function:        BYTE MACGet()
 * PreCondition:    ERDPT must point to the place to read from.
 * Output:          Byte read from the ENC28J60's RAM
 * Overview:        MACGet returns the byte pointed to by ERDPT and
 *                  increments ERDPT so MACGet() can be called again.  The
 *                  increment will follow the receive buffer wrapping boundary.
 *****************************************************************************/
unsigned char MACGet(){
    unsigned char Result;
    enc_chip_enable(); //ClearSPIDoneFlag();
	
	// Send the opcode and read a byte in two 8-bit operations
	enc_spi_transmit(ENC_READBUFF); //read buff
	Result = enc_spi_read();
    
    enc_chip_disable();
    return Result;
}//end MACGet

/******************************************************************************
 * Function:        WORD MACGetArray(BYTE *val, WORD len)
 * PreCondition:    SPI bus must be initialized (done in MACInit()).
 *                  ERDPT must point to the place to read from.
 * Input:           *val: Pointer to storage location
 *                  len:  Number of bytes to read from the data buffer.
 * Output:          Byte(s) of data read from the data buffer.
 * Overview:        Burst reads several sequential bytes from the data buffer
 *                  and places them into local memory.  With SPI burst support,
 *                  it performs much faster than multiple MACGet() calls.
 *                  ERDPT is incremented after each byte, following the same
 *                  rules as MACGet().
 *****************************************************************************/
unsigned int MACGetArray(unsigned char *val, unsigned int len){
    // Start the burst operation
    enc_chip_enable(); //ClearSPIDoneFlag();
    enc_spi_transmit(ENC_READBUFF); //read buff
    
    // Read the data
	unsigned int read = enc_spi_read_array(val,len);

    // Terminate the burst operation
    enc_chip_disable();

    return read;
}//end MACGetArray

/******************************************************************************
 * Function:        void MACPut(BYTE val)
 * PreCondition:    EWRPT must point to the location to begin writing.
 * Input:           Byte to write into the ENC28J60 buffer memory
 * Overview:        MACPut outputs the Write Buffer Memory opcode/constant
 *                  (8 bits) and data to write (8 bits) over the SPI.
 *                  EWRPT is incremented after the write.
 *****************************************************************************/
void MACPut(unsigned char val){
    enc_chip_enable(); //ClearSPIDoneFlag();
	enc_spi_transmit(ENC_WRITEBUFF);
	enc_spi_transmit(val);
    enc_chip_disable();
}//end MACPut

/******************************************************************************
 * Function:        void MACPutArray(BYTE *val, WORD len)
 * PreCondition:    EWRPT must point to the location to begin writing.
 * Input:           *val: Pointer to source of bytes to copy.
 *                  len:  Number of bytes to write to the data buffer.
 * Overview:        MACPutArray writes several sequential bytes to the
 *                  ENC28J60 RAM.  It performs faster than multiple MACPut()
 *                  calls.  EWRPT is incremented by len.
 *****************************************************************************/
void MACPutArray(unsigned char * val, unsigned int len){
    // Select the chip and send the proper opcode
    enc_chip_enable();//ClearSPIDoneFlag();
    enc_spi_transmit(ENC_WRITEBUFF);
    // Send the data, one byte at a time
    enc_spi_transmit_array(val, len);
    // Terminate the burst operation
    enc_chip_disable();
}//end MACPutArray

/******************************************************************************
 * Function:        void MACPutROMArray(ROM BYTE *val, WORD len)
 * PreCondition:    EWRPT must point to the location to begin writing.
 * Input:           *val: Pointer to source of bytes to copy.
 *                  len:  Number of bytes to write to the data buffer.
 * Overview:        MACPutArray writes several sequential bytes to the
 *                  ENC28J60 RAM.  It performs faster than multiple MACPut()
 *                  calls.  EWRPT is incremented by len.
 *****************************************************************************/
void MACPutROMArray(rom unsigned char * val, unsigned int len){
	MACPutArray((unsigned char*)val,len);
/*    volatile BYTE Dummy;

    // Select the chip and send the proper opcode
    ENC_CS_IO = 0;
    ClearSPIDoneFlag();
    ENC_SSPBUF = WBM;       // Send the Write Buffer Memory opcode
    WaitForDataByte();      // Wait until opcode/constant is transmitted.
    Dummy = ENC_SSPBUF;

    // Send the data
    while(len)
    {
        ENC_SSPBUF = *val;  // Start sending the byte
        val++;              // Increment after writing to ENC_SSPBUF to increase speed
        len--;              // Decrement after writing to ENC_SSPBUF to increase speed
        WaitForDataByte();  // Wait until byte is transmitted
        Dummy = ENC_SSPBUF;
    };

    // Terminate the burst operation
    ENC_CS_IO = 1;*/
}//end MACPutROMArray



/******************************************************************************
 * Function:        static void SendSystemReset(void)
 * Overview:        SendSystemReset sends the System Reset SPI command to
 *                  the Ethernet controller.  It resets all register contents
 *                  (except for ECOCON) and returns the device to the power
 *                  on default state.
 *****************************************************************************/
static void SendSystemReset(){
    //volatile BYTE Dummy;
    // Note: The power save feature may prevent the reset from executing, so
    // we must make sure that the device is not in power save before issuing
    // a reset.
    BFCReg(ENC_ECON2, 1<<ENC_PWRSV);

    // Give some opportunity for the regulator to reach normal regulation and
    // have all clocks running
	delay_ms(10);
	
    // Execute the System Reset command
    enc_chip_enable(); //ClearSPIDoneFlag();
    enc_spi_transmit(ENC_SYSRESET);
    enc_chip_disable();
    // Wait for the oscillator start up timer and PHY to become ready
    delay_ms(10);
}//end SendSystemReset

/******************************************************************************
 * Function:        REG ReadETHReg(BYTE Address)
 * PreCondition:    Bank select bits must be set corresponding to the register
 *                  to read from.
 * Input:           5 bit address of the ETH control register to read from.
 *                    The top 3 bits must be 0.
 * Output:          Byte read from the Ethernet controller's ETH register.
 * Overview:        ReadETHReg sends the 8 bit RCR opcode/Address byte over
 *                  the SPI and then retrives the register contents in the
 *                  next 8 SPI clocks.
 * Note:            This routine cannot be used to access MAC/MII or PHY
 *                  registers.  Use ReadMACReg() or ReadPHYReg() for that
 *                  purpose.
 *****************************************************************************/
 
 /*
	#define spi_stat ssp2stat
	#define spi_interrupt_flag SSP2IF
	#define spi_interrupt_reg pir3
	#define spi_buf ssp2buf*/

unsigned char ReadETHReg(unsigned char Address){//static
    unsigned char r;
    // Select the chip and send the Read Control Register opcode/address
    enc_chip_enable(); //ClearSPIDoneFlag();
    nop();
    enc_spi_transmit( ENC_READREG | (Address & ENC_ADDRESSMASK));
    r = enc_spi_read();
    enc_chip_disable();
    
    /*enc_chip_enable();
    
    clear_bit(pir3, SSP2IF);
	ssp2buf = Address & 0b11111;
	while (!test_bit(pir3, SSP2IF));
	clear_bit(pir3, SSP2IF);
	r = ssp2buf;
    ssp2buf=0;
    while (!test_bit(pir3, SSP2IF));
    clear_bit(pir3, SSP2IF);
    r = ssp2buf;
    enc_chip_disable();*/
    return r;
}//end ReadETHReg

/******************************************************************************
 * Function:        REG ReadMACReg(BYTE Address)
 * PreCondition:    Bank select bits must be set corresponding to the register
 *                  to read from.
 * Input:           5 bit address of the MAC or MII register to read from.
 *                    The top 3 bits must be 0.
 * Output:          Byte read from the Ethernet controller's MAC/MII register.
 * Overview:        ReadMACReg sends the 8 bit RCR opcode/Address byte as well
 *                  as a dummy byte over the SPI and then retrives the
 *                  register contents in the last 8 SPI clocks.
 * Note:            This routine cannot be used to access ETH or PHY
 *                  registers.  Use ReadETHReg() or ReadPHYReg() for that
 *                  purpose.
 *****************************************************************************/
static unsigned char ReadMACReg(unsigned char Address){
    unsigned char r;
    enc_chip_enable(); //ClearSPIDoneFlag();
    nop();
    enc_spi_transmit(ENC_READREG | (Address & ENC_ADDRESSMASK)); // Send the Read Control Register opcode and
    r = enc_spi_read(); // Send a dummy byte
    r = enc_spi_read();
    ENC_WRITEREG_DELAY
    enc_chip_disable();
    nop();
    return r;
}//end ReadMACReg


/******************************************************************************
 * Function:        ReadPHYReg
 * PreCondition:    SPI bus must be initialized (done in MACInit()).
 * Input:           Address of the PHY register to read from.
 * Output:          16 bits of data read from the PHY register.
 * Side Effects:    None
 * Overview:        ReadPHYReg performs an MII read operation.  While in
 *                  progress, it simply polls the MII BUSY bit wasting time
 *                  (10.24us).
 *****************************************************************************/
unsigned int ReadPHYReg(unsigned char Register){
    unsigned int Result;
    unsigned char low,high;
    unsigned char loops=0;

    // Set the right address and start the register read operation
    BankSel(ENC_MIREGADR);
    WriteReg(ENC_MIREGADR, Register);
    WriteReg(ENC_MICMD, 1<<ENC_MIIRD);

    // Loop to wait until the PHY register has been read through the MII
    // This requires 10.24us
    BankSel(ENC_MISTAT);
    while(ReadMACReg(ENC_MISTAT) & (1<< ENC_BUSY) && loops!=128){//after few 100s the BUSY flag should be cleared or we have comm. problems
		loops++;
		delay_us(1);
    }
	
    // Stop reading
    BankSel(ENC_MIREGADR);
    WriteReg(ENC_MICMD, 0x00);

    // Obtain results and return
    low = ReadMACReg(ENC_MIRDL);
    high = ReadMACReg(ENC_MIRDH);
	MAKESHORT(Result,low,high);
    BankSel(ENC_ERDPTL);    // Return to Bank 0

    return Result;
}//end ReadPHYReg

/******************************************************************************
 * Function:        void WriteReg(BYTE Address, BYTE Data)
 * PreCondition:    Bank select bits must be set corresponding to the register
 *                  to modify.
 * Input:           5 bit address of the ETH, MAC, or MII register to modify.
 *                    The top 3 bits must be 0.
 *                  Byte to be written into the register.
 * Overview:        WriteReg sends the 8 bit WCR opcode/Address byte over the
 *                  SPI and then sends the data to write in the next 8 SPI
 *                  clocks.
 * Note:            This routine is almost identical to the BFCReg() and
 *                  BFSReg() functions.  It is seperate to maximize speed.
 *                  Unlike the ReadETHReg/ReadMACReg functions, WriteReg()
 *                  can write to any ETH or MAC register.  Writing to PHY
 *                  registers must be accomplished with WritePHYReg().
 *****************************************************************************/
static void WriteReg(unsigned char Address,unsigned char Data){
    enc_chip_enable(); //ClearSPIDoneFlag();
    nop();
	enc_spi_transmit(ENC_WRITEREG | (Address & ENC_ADDRESSMASK)); // Send the opcode and address.
	enc_spi_transmit(Data);          // Send the byte to be writen.
	// For faster processors (dsPIC), delay for a few clock cycles to ensure 
	// the MAC/MII register write Chip Select hold time minimum of 210ns is met.
	ENC_WRITEREG_DELAY
	enc_chip_disable();
}//end WriteReg

/******************************************************************************
 * Function:        void BFCReg(BYTE Address, BYTE Data)
 * PreCondition:    Bank select bits must be set corresponding to the register
 *                    to modify.
 * Input:           5 bit address of the register to modify.  The top 3 bits
 *                    must be 0.
 *                  Byte to be used with the Bit Field Clear operation.
 * Overview:        BFCReg sends the 8 bit BFC opcode/Address byte over the
 *                  SPI and then sends the data in the next 8 SPI clocks.
 * Note:            This routine is almost identical to the WriteReg() and
 *                  BFSReg() functions.  It is separate to maximize speed.
 *                  BFCReg() must only be used on ETH registers.
 *****************************************************************************/
static void BFCReg(unsigned char Address,unsigned char Data){
    enc_chip_enable(); //ClearSPIDoneFlag();
    enc_spi_transmit(ENC_CLEARBIT | (Address & ENC_ADDRESSMASK)); // Send the opcode and address.
    enc_spi_transmit(Data);          // Send the byte to be writen.
    enc_chip_disable();
}//end BFCReg

/*{*****************************************************************************
 * Function:        void BFSReg(BYTE Address, BYTE Data)
 * PreCondition:    Bank select bits must be set corresponding to the register
 *                  to modify.
 * Input:           5 bit address of the register to modify.  The top 3 bits
 *                    must be 0.
 *                  Byte to be used with the Bit Field Set operation.
 * Overview:        BFSReg sends the 8 bit BFC opcode/Address byte over the
 *                  SPI and then sends the data in the next 8 SPI clocks.
 * Note:            This routine is almost identical to the WriteReg() and
 *                  BFCReg() functions.  It is separate to maximize speed.
 *                  BFSReg() must only be used on ETH registers.
 ****************************************************************************}*/
static void BFSReg(unsigned char Address, unsigned char Data){
    enc_chip_enable();//ClearSPIDoneFlag();
    enc_spi_transmit(ENC_SETBIT | (Address & ENC_ADDRESSMASK)); // Send the opcode and address.
    enc_spi_transmit(Data);          // Send the byte to be writen.
    enc_chip_disable();
}//end BFSReg


/******************************************************************************
 * Function:        WritePHYReg
 * PreCondition:    SPI bus must be initialized (done in MACInit()).
 * Input:           Address of the PHY register to write to.
 *                  16 bits of data to write to PHY register.
 * Side Effects:    Alters bank bits to point to Bank 3
 * Overview:        WritePHYReg performs an MII write operation.  While in
 *                  progress, it simply polls the MII BUSY bit wasting time.
 *****************************************************************************/
void WritePHYReg(unsigned char Register, unsigned int Data){
    // Write the register address
    BankSel(ENC_MIREGADR);
    WriteReg(ENC_MIREGADR, Register);

    // Write the data
    // Order is important: write low byte first, high byte last
    WriteReg(ENC_MIWRL, ((WORD_VAL)Data).v[0]); //produces much smaller code than the pointer stuff from microchip
    WriteReg(ENC_MIWRH, ((WORD_VAL)Data).v[1]);
	
    // Wait until the PHY register has been written
    BankSel(ENC_MISTAT);
    while(ReadMACReg(ENC_MISTAT) & (1<<ENC_BUSY)); //wait while writing to the PHYReg
    BankSel(ENC_ERDPTL);    // Return to Bank 0 (why?)
}//end WritePHYReg

/******************************************************************************
 * Function:        BankSel
 * Input:           Register address
 * Overview:        BankSel takes the high 3 bits of a register address and
 *                  changes the bank select bits in ETHCON1 to match.
 *****************************************************************************/
static void BankSel(unsigned char Register){
    BFCReg(ENC_ECON1, (1<<ENC_BSEL1) | (1<<ENC_BSEL0));
    BFSReg(ENC_ECON1, (Register >> 5) & 0b11); //small adjustment to microchips code, requires less memory since the upper 3 bits of register address is not used anyway and there are only 4 banks
}//end BankSel


/******************************************************************************
 * Function:        void MACPowerDown(void)
 * Overview:        MACPowerDown puts the ENC28J60 in low power sleep mode. In
 *                  sleep mode, no packets can be transmitted or received.
 *                  All MAC and PHY registers should not be accessed.
 * Note:            If a packet is being transmitted while this function is
 *                  called, this function will block until it is it complete.
 *                  If anything is being received, it will be completed.
 *****************************************************************************/
void MACPowerDown(void){
    // Disable packet reception
    BFCReg(ENC_ECON1, 1<<ENC_RXEN);

    // Make sure any last packet which was in-progress when RXEN was cleared
    // is completed
    while(ReadETHReg(ENC_ESTAT) & (1<<ENC_RXBUSY));

    // If a packet is being transmitted, wait for it to finish
    while(ReadETHReg(ENC_ECON1) & (1<<ENC_TXRTS));

    // Enter sleep mode
    BFSReg(ENC_ECON2, 1<<ENC_PWRSV);
}//end MACPowerDown


/******************************************************************************
 * Function:        void MACPowerUp(void)
 * Overview:        MACPowerUp returns the ENC28J60 back to normal operation
 *                  after a previous call to MACPowerDown().  Calling this
 *                  function when already powered up will have no effect.
 * Note:            If a link partner is present, it will take 10s of
 *                  milliseconds before a new link will be established after
 *                  waking up.  While not linked, packets which are
 *                  transmitted will most likely be lost.  MACIsLinked() can
 *                  be called to determine if a link is established.
 *****************************************************************************/
void MACPowerUp(void){
    // Leave power down mode
    BFCReg(ENC_ECON2, 1<<ENC_PWRSV);

    // Wait for the 300us Oscillator Startup Timer (OST) to time out.  This
    // delay is required for the PHY module to return to an operational state.
    while(!(ReadETHReg(ENC_ESTAT) & (1<<ENC_CLKRDY)));

    // Enable packet reception
    BFSReg(ENC_ECON1, 1<<ENC_RXEN);
}//end MACPowerUp

void MACSetAddress(MAC_ADDR * adr){
	enc_MAC_addr = *adr;  
}

MAC_ADDR * MACGetAddress(){
	return &enc_MAC_addr;
}

/******************************************************************************
 * Function:        void SetCLKOUT(BYTE NewConfig)
 * Input:           NewConfig - 0x00: CLKOUT disabled (pin driven low)
 *                              0x01: Divide by 1 (25 MHz)
 *                              0x02: Divide by 2 (12.5 MHz)
 *                              0x03: Divide by 3 (8.333333 MHz)
 *                              0x04: Divide by 4 (6.25 MHz, POR default)
 *                              0x05: Divide by 8 (3.125 MHz)
 * Overview:        Writes the value of NewConfig into the ECOCON register.
 *                  The CLKOUT pin will beginning outputting the new frequency
 *                  immediately.
 *****************************************************************************/
void SetCLKOUT(unsigned char NewConfig){
    BankSel(ENC_ECOCON);
    WriteReg(ENC_ECOCON, NewConfig);
    BankSel(ENC_ERDPTL);
}//end SetCLKOUT

/******************************************************************************
 * Function:        BYTE GetCLKOUT(void)
 * PreCondition:    SPI bus must be initialized (done in MACInit()).
 * Output:          BYTE - 0x00: CLKOUT disabled (pin driven low)
 *                         0x01: Divide by 1 (25 MHz)
 *                         0x02: Divide by 2 (12.5 MHz)
 *                         0x03: Divide by 3 (8.333333 MHz)
 *                         0x04: Divide by 4 (6.25 MHz, POR default)
 *                         0x05: Divide by 8 (3.125 MHz)
 *                         0x06: Reserved
 *                         0x07: Reserved
 * Overview:        Returns the current value of the ECOCON register.
 *****************************************************************************/
unsigned char GetCLKOUT(){
    unsigned char i;
    BankSel(ENC_ECOCON);
    i = ReadETHReg(ENC_ECOCON);
    BankSel(ENC_ERDPTL);
    return i;
}//end GetCLKOUT

/******************************************************************************
 * Function:        void SetRXHashTableEntry(MAC_ADDR DestMACAddr)
 *
 * PreCondition:    SPI interface must be initialized (done in MACInit()).
 *
 * Input:           DestMACAddr: 6 byte group destination MAC address to allow 
 *								 through the Hash Table Filter.  If DestMACAddr 
 *								 is set to 00-00-00-00-00-00, then the hash 
 *								 table will be cleared of all entries and the 
 *								 filter will be disabled.
 *
 * Output:          Sets the appropriate bit in the EHT* registers to allow 
 *					packets sent to DestMACAddr to be received and enables the 
 *					Hash Table receive filter (if not already).
 *
 * Side Effects:    None
 *
 * Overview:        Calculates a CRC-32 using polynomial 0x4C11DB7 and then, 
 *					using bits 28:23 of the CRC, sets the appropriate bit in 
 *					the EHT0-EHT7 registers.
 *
 * Note:            This code is commented out to save code space on systems 
 *					that do not need this function.  Change the 
 *					"#if STACK_USE_ZEROCONF_MDNS_SD" line to "#if 1" to 
 *					uncomment it, assuming you aren't using the Zeroconf module, 
 *					which requires mutlicast support and enables this function 
 *					automatically.
 *
 *					There is no way to individually unset destination MAC 
 *					addresses from the hash table since it is possible to have 
 *					a hash collision and therefore multiple MAC addresses 
 *					relying on the same hash table bit.  The stack would have 
 *					to individually store each 6 byte MAC address to support 
 *					this feature, which would waste a lot of RAM and be 
 *					unnecessary in most applications.  As a simple compromise, 
 *					you can call SetRXHashTableEntry() using a 
 *					00-00-00-00-00-00 destination MAC address, which will clear 
 *					the entire hash table and disable the hash table filter.  
 *					This will allow you to then re-add the necessary 
 *					destination address(es).
*
 *					This function is intended to be used when 
 *					ERXFCON.ANDOR == 0 (OR).
 *****************************************************************************/
#if defined(STACK_USE_ZEROCONF_MDNS_SD)
	void SetRXHashTableEntry(MAC_ADDR DestMACAddr){
		unsigned long CRC = {0xFFFFFFFF};
		unsigned char HTRegister;
		unsigned char i, j;
	
		// Select proper bank for ERXFCON and EHT0-EHT7 register access
		BankSel(ENC_ERXFCON);
	
		// Clear the Hash Table bits and disable the Hash Table Filter if a special 
		// 00-00-00-00-00-00 destination MAC address is provided.
		if((DestMACAddr.v[0] | DestMACAddr.v[1] | DestMACAddr.v[2] | DestMACAddr.v[3] | DestMACAddr.v[4] | DestMACAddr.v[5]) == (unsigned int)0x00){
			// Disable the Hash Table receive filter and clear the hash table
			BFCReg(ENC_ERXFCON, 1<<ENC_HTEN);
			for(i = (unsigned char)ENC_EHT0; i <= (unsigned char)ENC_EHT7; i++)
				WriteReg(i, 0x00);
		}else{
			// Calculate a CRC-32 over the 6 byte MAC address
			// using polynomial 0x4C11DB7
			for(i = 0; i < sizeof(MAC_ADDR); i++){
				unsigned char  crcnext;
				// shift in 8 bits
				for(j = 0; j < 8; j++){
					crcnext = 0;
					if(CRC.31)crcnext = 1;
					crcnext ^= (DestMACAddr.v[i] & 1);
	
					CRC <<= 1;
					if(crcnext)
						CRC ^= 0x4C11DB7;
					// next bit
					DestMACAddr.v[i] >>= 1;
				}
			}
	
			// CRC-32 calculated, now extract bits 28:23
			// Bits 25:23 define where within the Hash Table byte the bit needs to be set
			// Bits 28:26 define which of the 8 Hash Table bytes that bits 25:23 apply to
			i = ((DWORD_VAL)CRC).v[3] & 0x1F;
			HTRegister = (i >> 2) + (unsigned char)EHT0;
			i = (i << 1) & 0x06;
			i.0 = CRC.2;
	
			// Set the proper bit in the Hash Table
			BFSReg(HTRegister, 1<<i);
	
			// Ensure that the Hash Table receive filter is enabled
			BFSReg(ENC_ERXFCON, 1<<ENC_HTEN);
		}
		BankSel(ENC_ERDPTL);            // Return to Bank 0
	}
#endif

//if an error occurs from the MAC/Ethernet this function will be called, device should be reset
//src=1 = error reading from ENC28J60, SPI error,...
void enc_error(unsigned char src){
	serial_send_string("ENC ERROR!");
	serial_send_int(src);
	MACInit();
}