/* HEADER FILE FOR THE ENC28J60 ethernet module
   In case you want to use this module without the entire TCP/IP STACK 
   add stackdefs.h & Helpers.h to your project and define the following in stackconf.h:
   #define ETH_ENC28J60
	#define enc_tris_cs trisa.2 //chip select
	#define enc_tris_rs trisa.3 //reset device
	#define enc_cs porta.2 //chip select
	#define enc_rs porta.3 //reset device
	#define FULL_DUPLEX //for full duplex ethernet
	//size of RAM in the ENC28J60 module
	#define MAC_TX_BUFFER_SIZE			(unsigned int)(1500) //don't need more 
	#define RAMSIZE			((unsigned int)8*(unsigned int)1024) //total ammount of RAM
	//redirect the ENC28J60 SPI functions to the STACK spi read/write functions
	//in case you want to only use the ENC28J60 you adjust these functions to your own SPI read/write functions
	#define enc_spi_transmit(a) stack_spi_transmit(a) // void enc_spi_transmit(unsigned char data);
	#define enc_spi_transmit_array(a,b) stack_spi_transmit_array(a,b) // void enc_spi_transmit_array(unsigned char * data, unsigned int len);
	#define enc_spi_read() stack_spi_read() // unsigned char enc_spi_read();
	#define enc_spi_read_array(a,b) stack_spi_read_array(a,b) //returns nr of bytes written, must be able to handle NULL PTR!!! unsigned int! unsigned int enc_spi_read_array(unsigned char * data, unsigned int len);
*/



#ifndef _ENC28J60_H_
	#define _ENC28J60_H_
	#include <system.h>
	#include <string.h>
	#include <stdio.h>
	#include <stdlib.h>
	#include <memory.h>
	#include "stackconf.h"
	#include "..\stackdefs.h"
	#include "..\Helpers.h"
	

	#define TCP_ETH_RAM_SIZE 0 //not implemented, all TCP data is stored in the serial RAM
	#define TXSTART 		(RAMSIZE - (unsigned int)(MAC_TX_BUFFER_SIZE+1+18+7)) //- (unsigned int)TCP_ETH_RAM_SIZE - (unsigned int)RESERVED_HTTP_MEMORY - (unsigned int)RESERVED_SSL_MEMORY)
	#define RXSTART			((unsigned int)0)						// Should be an even memory address; must be 0 for errata
	#define	RXSTOP			((unsigned int)(TXSTART-2) | (unsigned int)0x0001)	// Odd for errata workaround
	#define RXSIZE			(RXSTOP-RXSTART+(unsigned int)1) //total ammount of RX memory
	#define BASE_TX_ADDR	(TXSTART + (unsigned int)1) //where tx memeory starts
	
	#define MAC_IP      	(unsigned char)(0x00)
	#define MAC_ARP     	(unsigned char)(0x06)
	#define MAC_UNKNOWN 	(unsigned char)(0xFF)
	
	
	//public functions to be used for accessing the MAC layer
	//unsigned char ReadETHReg(unsigned char Address);
	void MACInit();
	bool MACIsLinked();
	bool MACIsTxReady();
	void MACDiscardRx(void);
	unsigned int MACGetFreeRxSize();
	bool MACGetHeader(MAC_ADDR *remote, unsigned char * type);
	void MACPutHeader(MAC_ADDR *remote, unsigned char type, unsigned int dataLen);
	void MACFlush(void);
	void MACSetReadPtrInRx(unsigned int offset);
	unsigned int MACSetWritePtr(unsigned int address);
	unsigned int MACSetReadPtr(unsigned int address);
	unsigned int MACCalcRxChecksum(unsigned int offset, unsigned int len);
	unsigned int CalcIPBufferChecksum(unsigned int len);
	void MACMemCopyAsync(unsigned int destAddr, unsigned int sourceAddr, unsigned int len);
	bool MACIsMemCopyDone(void);
	unsigned char MACGet();
	unsigned int MACGetArray(unsigned char *val, unsigned int len);
	void MACPut(unsigned char val);
	void MACPutArray(unsigned char * val, unsigned int len);
	void MACPutROMArray(rom unsigned char * val, unsigned int len);
	void MACPowerDown(void);
	void MACPowerUp(void);
	void MACSetAddress(MAC_ADDR * adr);
	MAC_ADDR * MACGetAddress();
	
	//functions needed for SPI transmission/reception, adjust to your needs but don't remove/add arguments
	/*void enc_spi_transmit(unsigned char data);
	void enc_spi_transmit_array(unsigned char * data, unsigned int len);
	unsigned char enc_spi_read();
	unsigned int enc_spi_read_array(unsigned char * data, unsigned int len);
	void enc_error();
	
	//redirect to the main stack SPI functions
	inline void enc_spi_transmit(unsigned char data){
		stack_spi_transmit(data);
	}
		
	inline void enc_spi_transmit_array(unsigned char * data, unsigned int len){
		stack_spi_transmit_array(data, len);
	}
	
	inline unsigned char enc_spi_read(){
		return stack_spi_read();
	}
	
	inline unsigned int enc_spi_read_array(unsigned char * data, unsigned int len){
		stack_spi_read_array(data,len);
		return len;
	}*/
	
	//if an error occurs from the MAC/Ethernet this function will be called, device should be reset
	//src=1 = error reading from ENC28J60, SPI error,...
	void enc_error(unsigned char src);

	//delay used after writing to a register (must be min. of 210ns)
	#define ENC_WRITEREG_DELAY nop();nop();nop();nop();

#endif