//this file will be included by the IP stack and is used to configure the entire stack
//normaly it's not needed to edit the other stack files in order to make the project run
#ifndef _STACK_CONF_H
	#define _STACK_CONF_H
	//extern void pushcs(unsigned char sub); //was used for debugging
	//extern void popcs();//used for debugging
	//include some general files for spi, serial port, external ram
	//adjust the following lines to whatever files you want to use
	#include "sp.h"   //serial port header file (only needed for the debugging)
	#include "spi.h"  //SPI header file, defines several SPI read/write functions used in ENC28J60.h
	#include "sram.h" //header file for serial ram access
	#include "rtc.h"  //TCP requires tickget() and tickgetdiv256() this module will implement this
	
	extern void custom_network_task(); //this function will be called 
	
	#define NETWORK_INTERFACES 1 //only one interface supported right now
	
	//CONFIGURE ETHERNET MODULE HERE
	
	//Configure ENC28J60
	//Use ENC28J60 module
	#define ETH_ENC28J60
	#define enc_tris_cs trisa.1 //chip select
	#define enc_tris_rs trisa.2 //reset device
	#define enc_cs porta.0=1;porta.1 //chip select
	#define enc_rs porta.2 //reset device
	#define FULL_DUPLEX //for full duplex ethernet
	//size of RAM in the ENC28J60 module
	#define MAC_TX_BUFFER_SIZE			(unsigned int)(1500) //don't need more 
	#define RAMSIZE			((unsigned int)8*(unsigned int)1024) //total ammount of RAM
	//redirect the ENC28J60 SPI functions to the STACK spi read/write functions
	//in case you want to only use the ENC28J60 you adjust these functions to your own SPI read/write functions
	#define enc_spi_transmit(a) stack_spi_transmit(a) // void enc_spi_transmit(unsigned char data);
	#define enc_spi_transmit_array(a,b) stack_spi_transmit_array(a,b) // void enc_spi_transmit_array(unsigned char * data, unsigned int len);
	#define enc_spi_read() stack_spi_read() // unsigned char enc_spi_read();
	#define enc_spi_read_array(a,b) stack_spi_read_array(a,b) //returns nr of bytes written, must be able to handle NULL PTR!!! unsigned int! unsigned int enc_spi_read_array(unsigned char * data, unsigned int len);

	
	
	
	
	
	//config IP
	#define STACK_USE_IP			   //enables IP
	#define MY_IP_TTL           (100)  // Time-To-Live in hops
	//#define STACK_USE_IP_GLEANING	   // enables IP gleaning = getting IP address of the first received ICMP packet
	//#define STACK_USE_AUTOIP		   // enables auto ip in the range of 169.254/16 
	#define STACK_USE_NBNS		       //enables netBIOS names
	#define STACK_CLIENT_MODE		   //enables some client functions (needed for ARP,...)
	
	//config ICMP
	#define STACK_USE_ICMP //always turn this on if you use SERVER AND/OR CLIENT
	#define STACK_USE_ICMP_SERVER
	//#define STACK_USE_ICMP_CLIENT //not yet tested/implemented
	
	//Configure DHCP
	#define STACK_USE_DHCP_CLIENT      // enables DHCP
	
	//Configure DNS
	#define STACK_USE_DNS
	//#define STACK_USE_DYNAMICDNS_CLIENT  //enables DynDNS
	
	//Configure UDP
	#define STACK_USE_UDP //enable UDP
	#define MAX_UDP_SOCKETS 5 // First port number for randomized local port number selection
	#define LOCAL_UDP_PORT_START_NUMBER (4096u)
	#define LOCAL_UDP_PORT_END_NUMBER   (8192u)// Last port number for randomized local port number selection
	#define UDP_USE_TX_CHECKSUM //not yet tested seems that dhcp doesn't work if turned off and used together with MS MAC bridge software
	
	
	//Configure TCP
	#define STACK_USE_TCP                   //enables TCP
	#define LOCAL_PORT_START_NUMBER (1024u) // Starting port for client sockets
	#define LOCAL_PORT_END_NUMBER   (5000u) // End port for client sockets
	#define TCP_SOCKET_COUNT 5           //max number of TCP sockets
	#define TCP_OPTIMIZE_FOR_SIZE        //optimize TCP for code size (35% less code, 15% slower)
	#define TCP_SOCKET_RX_BUFFER 1600    //external RAM buffer size for one socket (receive)
	#define TCP_SOCKET_TX_BUFFER 1600    //external RAM buffer size for one socket (send)
	#define TCP_EX_RAM_BASE_ADDRESS 0    //the start address in external RAM (this can be any ram externaly connected to the pic (can also be ram from the enc28j60 module)
	#define TCP_EX_RAM_END_ADDRESS 16000 //max address of external TCP RAM = (TCP_SOCKET_RX_BUFFER+TCP_SOCKET_TX_BUFFER) * TCP_SOCKET_COUNT
	//#define TCP_DEBUG                    //enables TCP debugging (sends some stuff on the serial interface with serial_send_string() function)
	
	//configure time options required for TCP and all other blocking functions that support a timeout
	#define STACK_TICK_SECOND RTC_TICKS_PER_SECOND //number of ticks between 2 calls to Stack_TickGet() in 1 second. = timer increase frequency
	#define STACK_TICK_SECONDDIV256 (RTC_TICKS_PER_SECOND>>8) //same as above but devided by 256

	/************************************************/
	//configure functions used for SPI transmission to the ethernet module:
	
	//functions needed for SPI transmission/reception
	inline void stack_spi_transmit(unsigned char data){
		spi_transmit(data); //enter here a function that can transmit 1 byte on the SPI bus (function waits until transfer is done)
	}
	inline void stack_spi_transmit_array(unsigned char * data, unsigned int len){
		spi_transmit_array(data, len); //enter here a function that can transmit large arrays on the SPI bus (function waits until transfer is done)
	}
	inline unsigned char stack_spi_read(){
		return spi_read(); //define a function that reads one byte on the SPI bus and returns the value of it
	}
	inline unsigned int stack_spi_read_array(unsigned char * data, unsigned int len){
		spi_read_array(data,len);//must be able to handle NULL as data pointer!! else you get problems with TCP
		return len;				 //define a function that reads large arrays from the SPI bus and returns the total number of actual bytes written
	}

	//here you add external functions that enable the TCP/IP stack to read/write to external RAM needed for connection
	inline void stack_sram_init(){ //called when the stack is initialized (just before TCP is/should be loaded), can be used to prepare the sram module
		//we reserve memory
		sram_set_stack_start(((unsigned int)TCP_SOCKET_RX_BUFFER+(unsigned int)TCP_SOCKET_TX_BUFFER)*(unsigned int)TCP_SOCKET_COUNT+1);
	}
	inline void stack_copy_to_sram(unsigned int ptrDest, unsigned int ptrSource, unsigned int wLength){
		sram_write(ptrDest, (unsigned char *) ptrSource, wLength); //copies memory from PIC ram (ptrSource) to serial ram (ptrDest)
	}
	inline void stack_copy_from_sram(unsigned int ptrDest, unsigned int ptrSource, unsigned int wLength){
		sram_read(ptrSource,(unsigned char *) ptrDest,wLength); //copies memory from serial ram(PtrSource) to the PIC ram(ptrDest)
	}
	inline void stack_copy_sram_to_sram(unsigned int ptrDest, unsigned int PtrSource, unsigned int wLength){
		sram_copy(PtrSource, ptrDest, wLength); //for memory copy between 2 ram locations in the external (serial ram)
	}
	
	//functions used by the TCP.c file to allow timings
	inline unsigned long Stack_TickGet(){
		return rtc_get_tick(); //this function returns a 4 byte integer holding the ammount of clock ticks on a timer since restart
							   //this function is needed for the TCP stack to function correct
	}
	inline unsigned long Stack_TickGetDiv256(){
		return rtc_get_tick_div256(); //also returns clock ticks since startup but devided by 256 (for long timings)
	}
	//used for functions with a timeout
	//this should return the number of seconds since processor restart or number of seconds since 1970
	inline unsigned long Stack_GetTime(){
		return rtc_get_time();
	}
	
#endif